<?php

// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.


/**
 * Listaa kaikki netforum-modulin ilmentymät yhdessä kurssissa
 * 
 * Moodlen vaatima moduulin rakenteeseen kuuluva perussivu joka listaa
 * yksittäiseen kurssiin kuuluvat netforum-ilmentymät. Piilotetut ilmentymät
 * näytetään himmennettyinä niille joilla on oikeus nähdä ne, sekä tietenkin
 * piilotetaan niiltä joilla näkemisoikeuksia ei ole. Kustakin ilmentymästä
 * listataan nimi, kuvaus ja julkaistujen viestien yhteismäärä.
 *
 * @package   mod-netforum
 * @copyright 2009 Helsingin Yliopisto, Tietojenkäsittelytieteen laitos
 * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

require_once (dirname(dirname(dirname(__FILE__))).'/config.php');
require_once ("lib.php");


// Kurssi jonka instanssi listataan
// jos kurssia ei ole annettu oletaa moodlen globaalin kurssin
$id = optional_param('id', 0, PARAM_INT);

if ($id) {
    if (! $course = get_record('course', 'id', $id)) {
        error(get_string('cid_incorrect', 'netforum')); // "Course ID is incorrect"
    }
} else {
    if (! $course = get_site()) {
        error(get_string('no_toplevel', 'netforum')); // "Could not find a top-level course!"
    }
}

require_course_login($course);

add_to_log($course->id, 'netforum', 'view all', "index.php?id=$course->id", '');

// Haetaan lokalisaatiot
$strnetforums = get_string('modulenameplural', 'netforum');
$strnetforum  = get_string('modulename', 'netforum');

// Generoidaan »polkupalkki»
$navlinks = array();
$navlinks[] = array('name' => $strnetforums, 'link' => '', 'type' => 'activity');

$navigation = build_navigation($navlinks);

// Tulostetaan sivun otsakkeet
print_header_simple($strnetforums, '', $navigation, '', '', true, '', navmenu($course));


// Haetaan ilmentymien tiedot
if (! $netforums = get_all_instances_in_course('netforum', $course)) {
    notice(get_string('no_instances', 'netforum') , "../../course/view.php?id=$course->id"); // 'There are no instances of netforum'
    die;
}

// Generoidaan ilmentymistä taulukko

$strname  = get_string('name');
$strposts = get_string('numberofposts', 'netforum');
$strintro  = get_string('intro', 'netforum');

//$table->head  = array ($strname, $strintro,  $strposts);
$table->head  = array ($strname, $strintro);
$table->align = array ('left', 'left', 'left', 'left');



foreach ($netforums as $netforum) {
    if (!$netforum->visible) {
        //Jos ilmentymä on piilotettu näytetään himmennettynä
        $link = "<a class=\"dimmed\" href=\"view.php?id=$netforum->coursemodule\">$netforum->name</a>";
    } else {
        //Jos ilmentymä on näkyvissä näytetään normaalisti
        $link = "<a href=\"view.php?id=$netforum->coursemodule\">$netforum->name</a>";
    }
    $info = get_record("netforum", "id", intval($netforum->id))->intro;
    
    $table->data[] = array ($link, $info);
 
}

// Tulostetaan generoitu taulukko
print_heading($strnetforums);
print_table($table);

// Sivun varsinainen sisältö päättyy, tulostetaan lopputekstit

print_footer($course);

?>