<?php

// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

 /**
* Netforumin kirjastofunktiot
*
* @author    Mikko Johansson, Juho Kilpikoski, Tevje Olin, Max Pagels, Aleksi Toivanen
* @package   mod-netforum
* @copyright 2009 Helsingin Yliopisto, Tietojenkäsittelytieteen laitos
* @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
*/


require_once(dirname(dirname(dirname(__FILE__))).'/config.php');
require_once($CFG->libdir.'/filelib.php');

define ('NETFORUM_AGGREGATE_NONE', 0);
define ('NETFORUM_AGGREGATE_AVG', 1);
define ('NETFORUM_CHECKBOX_WIDTH', '20px');
define ('NETFORUM_NOTE_EXCERPT_LENGTH', 200);
define ('NETFORUM_DEFAULT_PAGE_SIZE', 20);
define ('NETFORUM_UNSET_POST_RATING', -999);


define ('NETFORUM_HELPTAB', 0);
define ('NETFORUM_TAB1', 1);
define ('NETFORUM_TAB2', 2);
define ('NETFORUM_TAB3', 3);
define ('NETFORUM_TAB4', 4);
define ('NETFORUM_TAB5', 5);
define ('NETFORUM_TAB6', 6);


/*
* author: Max Pagels
* funktio: netforum_install()
* syöte: -
* palauttaa: true, ei tehdä ylimääräistä moduulin asennuksen yhteydessä
*/
function netforum_install() {
    return true;
}

/*
* author: Max Pagels
* funktio: netforum_add_instance($netforum)
* syöte: $netforum-objekti
* palauttaa: instanssin id:n, jos objekti sisältää kaikki vaaditut tiedot jolla
*            instanssin luominen onnistuu, muuten false
*/
function netforum_add_instance($netforum) {
	global $CFG;
    $netforum->timemodified = time();

    if(empty($netforum->assessed)) {
        $netforum->assessed = 0;
    }

    if(empty($netforum->ratingtime) or empty($netforum->assessed)) {
        $netforum->assesstimestart = 0;
        $netforum->assesstimefinish = 0;
    }
    
    if(!$netforum->id = insert_record('netforum', $netforum)) {
        return false;
    }
	
    $netforum = stripslashes_recursive($netforum);
    netforum_grade_item_update($netforum);

    return $netforum->id;
}

/*
* author: Max Pagels
* funktio: netforum_update_instance($netforum)
* syöte: $netforum-objekti
* palauttaa: true, jos objekti sisältää kaikki vaaditut tiedot jolla
*            instanssin päivittäminen onnistuu, muuten false
*/
function netforum_update_instance($netforum) {
    $netforum->timemodified = time();
    $netforum->id = $netforum->instance;
    
    if(empty($netforum->assessed)) {
        $netforum->assessed = 0;
    }

    if(empty($netforum->ratingtime) or empty($netforum->assessed)) {
        $netforum->assesstimestart = 0;
        $netforum->assesstimefinish = 0;
    }
    
    $old = get_record('netforum', 'id', $netforum->id);
    $netforum = stripslashes_recursive($netforum);
    
    if ($old->scale<>$netforum->scale) {
        global $CFG;
        execute_sql("DELETE from {$CFG->prefix}netforum_ratings
                            WHERE postid in (SELECT id FROM {$CFG->prefix}netforum_post
                                             WHERE netforumid=$netforum->id)");
        netforum_grade_item_update($netforum, 'reset');
    }
    else {
        netforum_grade_item_update($netforum);
    }
    
    if(!update_record('netforum', $netforum)) {
        error('Could not update the netforum instance');
    }
    
    return true;
}

function netforum_delete_instance($id) {
    global $CFG;
    require_once($CFG->libdir.'/gradelib.php');
    
    if (!$netforum = get_record('netforum', 'id', $id)) {
        return false;
    }

    if(!execute_sql("DELETE from {$CFG->prefix}netforum_note
                            WHERE postid in (SELECT id FROM {$CFG->prefix}netforum_post
                                             WHERE netforumid=$netforum->id)")) {
        return false;
    }
    
    if(!execute_sql("DELETE from {$CFG->prefix}netforum_read
                            WHERE postid in (SELECT id FROM {$CFG->prefix}netforum_post
                                             WHERE netforumid=$netforum->id)")) {
        return false;
    }
    
    if(!execute_sql("DELETE from {$CFG->prefix}netforum_ratings
                            WHERE postid in (SELECT id FROM {$CFG->prefix}netforum_post
                                             WHERE netforumid=$netforum->id)")) {
        return false;
    }
    
    if(!execute_sql("DELETE from {$CFG->prefix}netforum_web
                            WHERE child in (SELECT id FROM {$CFG->prefix}netforum_post
                                            WHERE netforumid=$netforum->id)")) {
        return false;
    }
    
    if(!delete_records('netforum_post', 'netforumid', $id)) {
        return false;
    }
    
    if(!delete_records('netforum', 'id', $netforum->id)) {
        return false;
    }
    
    netforum_grade_item_delete($netforum);
    
    return true;
}

/*
* author: Max Pagels
* funktio: netforum_user_outline($course, $user, $mod, $netforum)
* syöte: -
* palauttaa: -
*/
function netforum_user_outline($course, $user, $mod, $netforum) {
   
}

/*
* author: Max Pagels
* funktio: netforum_user_complete($course, $user, $mod, $netforum)
* syöte: -
* palauttaa: -
*/
function netforum_user_complete($course, $user, $mod, $netforum) {
    
}

/*
* author: Max Pagels
* funktio: netforum_uninstall()
* syöte: -
* palauttaa: true, ei tehdä ylimääräistä moduulin poiston yhteydessä
*/
function netforum_uninstall() {
    return true;
}

/*
* author: Max Pagels
* funktio: netforum_get_aggregate_types()
* syöte: -
* palauttaa: taulukon, jossa kaksi vakiota jota käytetään kun luodaan netforumista
*            instassi: NETFORUM_AGGREGATE_NONE (ei arvostella tekstejä) ja
*            NETFORUM_AGGREGATE_AVG (opettajat arvostelee tekstejä)
*/
function netforum_get_aggregate_types() {
    return array (NETFORUM_AGGREGATE_NONE  => get_string('aggregatenone', 'netforum'),
                  NETFORUM_AGGREGATE_AVG   => get_string('aggregateavg', 'netforum'));
}

/**
* @author Max Pagels
*/
function netforum_get_user_grades($netforum, $userid=0) {
    global $CFG;
    
    $user = $userid ? "AND u.id = $userid" : "";
    $aggregatetype = $netforum->assessed;
    
    $query = "SELECT u.id, u.id AS userid, AVG(nr.rating) AS rawgrade
              FROM {$CFG->prefix}user u,
                   {$CFG->prefix}netforum_post np,
                   {$CFG->prefix}netforum_ratings nr
              WHERE u.id = np.userid
              AND nr.postid = np.id
              AND nr.userid != u.id
              AND np.netforumid = $netforum->id $user
              GROUP BY u.id";
    
    return get_records_sql($query);
}

/**
* @author Max Pagels
*/
function netforum_get_number_of_notes($netforum, $userid, $postid=0, $citation=true) {
    global $CFG, $USER;

    if($citation) {
        $link = 1;
    }
    else {
        $link = 0;
    }

    if($postid) {
        $note = "AND p.id=$postid";
    }
    else {
        $note = '';
    }

    return count_records_sql("SELECT COUNT(DISTINCT n.id)
                              FROM {$CFG->prefix}netforum_note n,
                                   {$CFG->prefix}netforum_post p
                              WHERE n.userid=$USER->id
                              AND n.postid=p.id
                              AND p.netforumid=$netforum->id
                              AND n.link=$link
                              $note");
}

function netforum_get_checkbox_javascript() {
    return '<script type="text/javascript">
            //<![CDATA[
            function checkall(inputname) {
                var inputs = document.getElementsByName(inputname);
                for(var i = 0; i < inputs.length; i++) {
                    inputs[i].checked = true;
                }
            }
            function checknone(inputname) {
                var inputs = document.getElementsByName(inputname);
                for(var i = 0; i < inputs.length; i++) {
                    inputs[i].checked = false;
                }
            }
            //]]>
            </script>';
}

/**
* @author Max Pagels
*/
function netforum_add_user_citation($postid, $userid) {
    $citation = new object();
    $citation->postid = $postid;
    $citation->userid = $userid;
    $citation->link = 1;
    return insert_record('netforum_note', $citation);
}

function netforum_get_user_citations($netforum, $userid, $notes=false, $postid=0) {
    global $CFG;
    
    if(!$notes) {
        $link = 0;
    }
    else {
        $link = 1;
    }
    
    if(!$postid) {
        return get_recordset_sql("SELECT n.*
                                  FROM {$CFG->prefix}netforum_note n,
                                       {$CFG->prefix}netforum_post p
                                  WHERE n.userid=$userid
                                  AND n.postid=p.id
                                  AND p.netforumid=$netforum->id
                                  AND n.link=$link");
    }
    
    else {
        return get_recordset_sql("SELECT n.*
                                  FROM {$CFG->prefix}netforum_note n,
                                       {$CFG->prefix}netforum_post p
                                  WHERE n.userid=$userid
                                  AND n.postid=$postid
                                  AND n.postid=p.id
                                  AND p.netforumid=$netforum->id
                                  AND n.link=$link");
    }
}

function netforum_citation_delete($citationids) {
    if(!is_array($citationids)) {
        $citationids = array($citationids);
    }
    
    $success = true;
    foreach($citationids as $citationid) {
        if(is_null($citationid)) {
            continue;
        }
        if(!delete_records('netforum_note', 'id', $citationid)) {
            $success = false;
        }
    }
    
    return $success;
}

/*
* author: Max Pagels
* funktio: netforum_grade_item_update($forum, $grades=NULL)
* syöte: $netforum-objekti jossa extrana cmidnumber (course module id number) sekä
*        valinnainen taulukko/objekti $grades: 'reset' resetoi arvosanat moodlen
*        gradebookissa
* palauttaa: 0 jos netforumin grade item lisättiin onnistuneesti
*/
function netforum_grade_item_update($netforum, $grades=NULL) {
    global $CFG;
    
    if (!function_exists('grade_update')) {
        require_once($CFG->libdir.'/gradelib.php');
    }
    
    $params = array('itemname'=>$netforum->name, 'idnumber'=>$netforum->cmidnumber);
    
    if(!$netforum->assessed or $netforum->scale == 0) {
        $params['gradetype'] = GRADE_TYPE_NONE;
    }
	
	else if ($netforum->scale > 0) {
	    $params['gradetype'] = GRADE_TYPE_VALUE;
	    $params['grademax']  = $netforum->scale;
        $params['grademin']  = 0;
	}
	
	else if ($netforum->scale < 0) {
        $params['gradetype'] = GRADE_TYPE_SCALE;
        $params['scaleid']   = -$netforum->scale;
	}
	
	if ($grades  === 'reset') {
	    $params['reset'] = true;
	    $grades = NULL;
	}
	
	return grade_update('mod/netforum', $netforum->course, 'mod', 'netforum',
	                     $netforum->id, 0, $grades, $params);
}

/*
* author: Max Pagels
* funktio: netforum_grade_item_delete($netforum)
* syöte: $netforum-objekti
* palauttaa: $grade_item-objektin
*/
function netforum_grade_item_delete($netforum) {
    global $CFG;
    require_once($CFG->libdir.'/gradelib.php');
    
    return grade_update('mod/netforum', $netforum->course, 'mod', 'netforum',
                         $netforum->id, 0, NULL, array('deleted'=>1));
}

function netforum_update_grades($netforum, $userid=0, $nullifnone=true) {
    global $CFG;
    require_once($CFG->libdir.'/gradelib.php');

    if ($grades = netforum_get_user_grades($netforum, $userid)) {
        netforum_grade_item_update($netforum, $grades);
    }
    
    else if($userid and $nullifnone) {
        $grade = new object();
        $grade->userid = $userid;
        $grade->rawgrade = NULL;
        netforum_grade_item_update($netforum, $grade);
    }
    
    else {
        netforum_grade_item_update($netforum);
    }
}

/**
 * Hakee kaikki yhteen netforumilmentymään liittyvät kirjoitukset.
 *
 * @author Juho Kilpikoski
 * @param $netforumid haettavan netforumilmentymän id
 * @return array palautetaan taulukko, jonka alkiot ovat kyselyn tulosrivejä vastaavia olioita, tai false.
 *   Olioiden sisältämät tiedot ovat id, subject, userid, released, firstname, lastname ja
 *   adjacent (naapurien määrä). False palautetaan virhetilanteessa
 *   
 */
function netforum_get_posts($netforumid, $userid = false) {
    global $CFG;
    
    $id = intval($netforumid);
    
    
    if($userid === false){
        $userclause = '';    
    } else {
        $userclause = "AND u.id = " . intval($userid);
    }
    
    
    $sql = "(
SELECT p.id, p.subject, p.userid, p.released, u.firstname, u.lastname, COUNT( p.id ) AS adjacent
FROM {$CFG->prefix}netforum_post p, {$CFG->prefix}netforum_web w, {$CFG->prefix}user u
WHERE p.netforumid = $id 
AND (p.id = w.parent OR p.id = w.child) 
AND u.id = p.userid
AND p.released != 0
$userclause
GROUP BY p.id
)
UNION (

SELECT p.id, p.subject, p.userid, p.released, u.firstname, u.lastname, 0 AS adjacent
FROM {$CFG->prefix}netforum_post p, {$CFG->prefix}user u
WHERE p.netforumid = $id  
AND p.released != 0
AND p.id NOT 
IN (

SELECT w.parent
FROM {$CFG->prefix}netforum_web w, {$CFG->prefix}netforum_post p
WHERE w.parent = p.id
AND p.released != 0
)
AND p.id NOT 
IN (

SELECT w.child
FROM {$CFG->prefix}netforum_web w, {$CFG->prefix}netforum_post p
WHERE w.child = p.id
AND p.released != 0
)
AND u.id = p.userid
AND p.released != 0
$userclause
)
ORDER BY released DESC";



    return get_records_sql($sql);
}

/**
 * Hakee yksittäisen netforumilmentymän suurimman naapuriluvun
 *
 * @author Juho Kilpikoski
 * @param $netforumid haettavan netforumilmentymän id
 * @return int Suurin naapuriluku. 0 jos ei lainkaan viestejä.
 *   Virherilanteissa palautetaan myös 0.
 */
function netforum_count_most_connected($netforumid) {
    global $CFG;
    $id = intval($netforumid);
    $sql = "SELECT p.id, COUNT( p.id ) AS adjacent
FROM {$CFG->prefix}netforum_post p, {$CFG->prefix}netforum_web w
WHERE p.netforumid = $id
AND p.id = w.parent
AND p.released != 0 
GROUP BY p.id
ORDER BY adjacent DESC
";

    $query = get_record_sql($sql, true);
    
    // Palautetaan aina kokonaisluku
    if($query === false){
        return 0;
    } else {
        return $query->adjacent;
    }
}
/**
 * Palauttaa otsikkopilveen kuuuvan viestin otsikon verkottumisluokan
 * Otsikkopilven verkottumisluokat lasketaan aina tällä funktiolla.
 * Jos verkottumisluokkien määräytisperusteita halutaan muuttaa, riittää
 * siis muuttaa ainoastaa tätä funktiota. Funktiossa on käytetty 
 * yksinkertaista tasajakoa jossa  kuhunkin luokkaan kuulu 20%-yksikön
 * kaistaleelle osuvat suhdeluvut.
 * 
 * Jos parametreissä on määritetty vähiten verkottuneet piilotettaviksi,
 * funktio asettaa alimpaan verkottumisluokkan sijasta piilotettu-luokan.
 *
 * @author Juho Kilpikoski
 * @param $adjacent viestin naapurien määrä
 * @param $maxadjacent suurin naapurien määrä
 * @param $hidelevel5 piilotetaanko »turhimmat» viestit
 * @return string viestin otsikkon css-luokka
 */
function netforum_cloud_class($adjacent, $maxadjacent, $hidelevel5){
    
    // Jos netforumissa ei ole lainkaan kaaria tai arvot eivät ole validit
    // kuuluu viesti automaattisesti alimpaan luokkaan.
    // Piilotusta ei kuitenkaan suoriteta! Pilvessä näkyy jotakin
    // vaikka ei olisi lainkaan kaaria
    if($maxadjacent <= 0 || $adjacent < 0){
        return "mod-netforum-cloud-class5";
    }

    // Muutoin lasketaan suhdeluku ja valitaan luokka sen perusteella.
    $value = $adjacent * 1.0 / $maxadjacent;

    if($value < 0.2){
        if($hidelevel5){
            return "mod-netforum-cloud-hidden";
        } else {
            return "mod-netforum-cloud-class5";
        }
    }
    if($value < 0.4){
        return "mod-netforum-cloud-class4";
    }
    if($value < 0.6){
        return "mod-netforum-cloud-class3";
    }
    if($value < 0.8){
        return "mod-netforum-cloud-class2";
    }
    return "mod-netforum-cloud-class1";
}

/**
 * Palauttaa viestin korostamiseen käytettävän css-luokan tai sanallisen kuvauksen mikäli sellainen on olemassa
 * 
 * @author Juho Kilpikoski
 * @param $postid viestin id
 * @param $writerid viestin kiroittajan id
 * @parma $class palautetaanko css-korostusluokan nimi (true) vai sanallinen kuvaus (false) 
 * @return luokan nimi, sanallinen kuvaus tai tyhjä
 */
function netforum_highlight($postid, $writerid, $class=true){
    global $USER;

    if(!isguestuser() ){
        if($USER->id === $writerid){
            if($class){
                return 'mod-netforum-own-post';
            } else {
                return get_string('own_post', 'netforum');
            }
        }
        $times = netforum_has_read_post_times($USER->id, $postid);
        
        if($times === 0){
            if($class){
                return 'unread';
            } else {
                return get_string('unread', 'netforum');
            }
        }
    }
    // guestille ei highlightata mitään
    return '';
}

/**
 * Tulostaa yksittäisestä viestistä joko taulukko tai
 * graafimuotoisen naapuriesityksen riippuen siitä onko
 * dotdir määritelty asetuksissa tyhjäksi vai ei.
 * Jos dotdir on määrittelemätön tulostaa vastaavan
 * virheilmoituksen.
 * 
 * @author Juho Kilpikoski
 * @param $postid viesti josta esitys tuotetaan
 * @return unknown_type
 */
function netforum_print_neighbour_block($postid){
    global $CFG;
    if(!isset($CFG->netforum_dotdir)){
        return;
    }else if($CFG->netforum_dotdir !== ''){
        print_heading_with_help(get_string('parentsandchildren','netforum'), 'neighbourhoodgraph', 'netforum');
        if(!isset($CFG->netforum_shrinkgraphs)){
            error(get_string('settings_unset', 'netforum') . ' (netforum_shrinkgraphs)');
        } else {
            if($CFG->netforum_shrinkgraphs){
                netforum_generate_neighbour_web('cmapx', $postid, false, true);

                echo "<div class=\"mod-netforum-center\"><img src=\"neighbourgraph.php?fmt=png&amp;id={$postid}&amp;shrink=1\" alt=\"".
             	"\" usemap=\"#postgraph\"/></div>";
                
            } else {
                netforum_generate_neighbour_web('cmapx', $postid, false);

                echo "<div style=\"\" class=\"mod-netforum-center mod-netforum-div-center\" id=\"mod-netforum-scrollbox-graph\">".
             	"<img src=\"neighbourgraph.php?fmt=png&amp;id={$postid}\" alt=\"".
             	"\" usemap=\"#postgraph\"/></div>";
            }
            echo '<p class="mod-netforum-center">'.get_string('neighbourgaphlegend', 'netforum') .'</p>';
        }
    } else {
        netforum_generate_neighbour_table($postid);
    }
}

/**
 * Tulostaa yksittäisen viestin vanhemmista ja lapsista taulukkoesityksen.
 *
 * @author Juho Kilpikoski
 * @param $postid viesti josta esitys tuoteteaan
 * @return ei mitään
 */
function netforum_generate_neighbour_table($postid){
    global $CFG;
    $pid = intval($postid);
    
    $childrensql = "SELECT p.id, p.userid, p.subject, p.released, u.firstname, u.lastname
FROM {$CFG->prefix}netforum_post p, {$CFG->prefix}netforum_web w, {$CFG->prefix}user u
WHERE w.child = p.id
AND w.parent = $pid
AND p.userid = u.id
AND p.released != 0";
    $parentssql = "SELECT p.id, p.userid, p.subject, p.released, u.firstname, u.lastname
FROM {$CFG->prefix}netforum_post p, {$CFG->prefix}netforum_web w, {$CFG->prefix}user u 
WHERE w.parent = p.id
AND w.child = $pid
AND p.userid = u.id
AND p.released != 0";
    
    $parents = get_records_sql($parentssql);
    
    $children = get_records_sql($childrensql);
    
    $netforumid = netforum_get_id($postid);
    $id = netforum_get_cmid($netforumid);
    
    if($parents){
        $parentshtml = '<ul>';
        foreach($parents as $row){
            $parentshtml .= "<li><a href=\"post.php?id={$id}&amp;post={$row->id}\">{$row->subject}</a></li>";
        }
        $parentshtml .= '</ul>';
    } else {
        $parentshtml = '<p>' . get_string('no_messages', 'netforum') . '</p>';
    }
    if($children){
        $childrenhtml = '<ul>';
        foreach($children as $row){
            $childrenhtml .= "<li><a href=\"post.php?id={$id}&amp;post={$row->id}\">{$row->subject}</a></li>";
        }
        $childrenhtml .= '</ul>';
    } else {
        $childrenhtml = '<p>' .get_string('no_messages', 'netforum') . '</p>';
    }

    $table->head  = array (get_string('parents', 'netforum'),
                           get_string('children', 'netforum'));
    $table->align = array ('left', 'left', 'left', 'left', 'left');
    $table->data[] = array ($parentshtml, $childrenhtml);
    print_table($table);
    
}

/**
 * Generoi haluttuun viestiin liittyvän naapuriverkon halutussa muodossa
 * Tuetut muodot ovat source, cmap, png ja pdf
 *
 * @author Juho Kilpikoski
 * 
 * @param $fmt tuotettavan tulostiedoston tyyppi
 * @param $postid viestin id
 * @param $return palautetaanko tulos vai kirjoitetaanko suoraan
 * @param $shrink kutistetaanko kuvan kokoa
 * @return mixed joko tulos tai  true kirjoitettaessta tai false virhetilanteessa
 */
function netforum_generate_neighbour_web($fmt, $postid, $return=false, $shrink=false) {
    global $CFG;
    
    // Jos polkumuuttuja on asettamaton tai tyhjä ei tehdä mitään
    if(!isset($CFG->netforum_dotdir) || $CFG->netforum_dotdir === ''){
        return false;
    }
    if(!isset($CFG->netforum_dotfont)){
        error(get_string('settings_unset', 'netforum') . ' (netforum_dotfont)');
    }

    $pid = intval($postid);

    $netforumid = netforum_get_id($postid);
    if($shrink){
        $dotsource = "digraph postgraph{ rankdir=\"LR\" size=\"8,100\" ratio=\"expand\" \n node[fontname=\"{$CFG->netforum_dotfont}\"]";
    } else {
        $dotsource = "digraph postgraph{ \n  fontsize=10.0\n node[fontname=\"{$CFG->netforum_dotfont}\"]";
    }
    
    // Haetaan solmujen tiedot
    $sql = "(
-- Keskusnoodi
SELECT p.id, p.userid, p.subject, p.released, u.firstname, u.lastname,  '' AS parent , '' AS child
FROM {$CFG->prefix}netforum_post p, {$CFG->prefix}user u
WHERE p.id = $pid 
AND  p.userid = u.id
)
UNION
-- Keskusnoodin lapset 
(
SELECT p.id, p.userid, p.subject, p.released, u.firstname, u.lastname, $pid AS parent , p.id AS child
FROM {$CFG->prefix}netforum_post p, {$CFG->prefix}netforum_web w, {$CFG->prefix}user u
WHERE w.child = p.id
AND w.parent = $pid
-- Ei luonnoskirjoituksia lapsiksi
AND w.value != 0
AND  p.userid = u.id
)
UNION
-- Keskusnoodin vanhemmat
(
SELECT p.id, p.userid, p.subject, p.released, u.firstname, u.lastname, p.id AS parent , $pid
FROM {$CFG->prefix}netforum_post p, {$CFG->prefix}netforum_web w, {$CFG->prefix}user u 
WHERE w.parent = p.id
AND w.child = $pid
AND p.userid = u.id

)";


    $nbrs = get_records_sql($sql);
    if($nbrs === false){
        $dotsource .= "bgcolor=red id0[label=\"ERROR: NO NODES (postid: {$pid})\"]";
    }else{
        $id = netforum_get_cmid($netforumid);

        // Naapurien väliset keskinäiset kaaret haetaan ensimmäisestä hausta genreoidulla haulla; ruumaa mutta toimii
        $parents = '( w.parent = ' . $pid;
        $children = '( w.child = ' . $pid;
        // Generoidaan dotsourceen solmut, käytetään record-rakennetta
                
        foreach($nbrs as $row){
            // lisätään tiedot lukemattomuudesta/omuudesta
            $highlight = netforum_highlight($row->id, $row->userid, false);
            $released = '';
            if($row->released == 0){
                $released = get_string('thisisdraft', 'netforum');
            } else {
                $released = userdate($row->released);
            }
            
            $tmpuser = new object();
            $tmpuser->firstname = $row->firstname;
            $tmpuser->lastname = $row->lastname;
            $name = fullname($tmpuser);
            
            if($highlight != ''){
                $highlight = '<tr><td align="left"><font color="dimgray">'.$highlight.'</font></td></tr>';
            }
            // Näytetään tärkeimmät tiedot myös tooltipissä (hyödyllistä kutistetuissa graafeissa)
            $tooltip = preg_replace('/\s+/', ' ', trim($row->subject)) . ' – ' . $name . ' ('.$released.')';
            // Naapurit
            if($row->parent != ""){
                
                $dotsource .=	"id{$row->id} [ shape=box label=<<table href=\"post.php?id={$id}&amp;post={$row->id}\" ";
                $dotsource .= "border=\"0\" cellborder=\"0\" cellpadding=\"0\" CELLSPACING=\"0\" >";
                $dotsource .= "<tr><td align=\"left\"><font color=\"blue\">";
                // Pilkotaan ylipitkiä otsikoita jotta kuva pysyy tiiviimpänä
                $dotsource .= wordwrap(trim($row->subject), 36, '</font></td></tr>\n<tr><td align="left"><font color="blue">');
                $dotsource .= "</font></td></tr><tr><td align=\"left\">{$name}</td></tr><tr><td align=\"left\">";
                $dotsource .= $released . '</td></tr>'.$highlight.'</table>> tooltip="'.$tooltip.'"]'."\n";
                $dotsource .= "id{$row->child}->{ id{$row->parent} }[color=\"red\", len=1.00];\n";
            // keskussolmu
            } else {
                $dotsource .= 'id' . $row->id .'[ root=true shape=box style="filled, rounded" label=<<table border="0"'.
                              ' cellborder="0" cellpadding="0" CELLSPACING="0" >';
                $dotsource .= '<tr><td align="left"><font color="red">';
                $dotsource .= wordwrap(trim($row->subject), 36, '</font></td></tr>\n<tr><td align="left"><font color="red">');
                $dotsource .= '</font></td></tr><tr><td align="left">' . $name .'</td></tr><tr><td align="left">';
                $dotsource .= $released . '</td></tr>'.$highlight.'</table>> tooltip="'.$tooltip.'"]'."\n";

            }
        }

        // Haetaan puuttuvien kaarien tiedot
        $sql3 = "
      	SELECT parent, child 
      	FROM {$CFG->prefix}netforum_web w 
      	WHERE  parent IN (
      		-- lista naapureista
      		SELECT p.id
            FROM {$CFG->prefix}netforum_post p, {$CFG->prefix}netforum_web w
            WHERE 
            ((w.child = p.id AND w.parent = $pid)
            OR
            (w.child = $pid AND w.parent = p.id))
            AND w.value != 0
      	) AND child IN (
      		-- lista naapureista
      		SELECT p.id
            FROM {$CFG->prefix}netforum_post p, {$CFG->prefix}netforum_web w
            WHERE 
            ((w.child = p.id AND w.parent = $pid)
            OR
            (w.child = $pid AND w.parent = p.id))
            AND w.value != 0
      	)";
        $adja = get_records_sql($sql3);
        // Generoidaan kaaret
        if($adja !== false){
            foreach ($adja as $row) {
                $dotsource .= "id{$row->child}->{ id{$row->parent} }[color=\"red\", len=1.00];\n";
            }
        }
    }// $nbrs !== false

    $dotsource .= '}';

    // sorsalistaus debuggausta varten
    if($fmt == 'source'){
        if($return){
            return $dotsource;
        } else {
            echo $dotsource;
            return true;
        }
    }

    // Sorsa on valmis, siirrytään graafin generointiin

    // Luodaan temp-filu sorsalle
    $dotfile = tempnam(sys_get_temp_dir(), "netforum-");

    $fp = fopen($dotfile, 'w');

    if ($fp) {

        fwrite($fp, $dotsource);
        fclose($fp);
    } else {
        unlink($dotfile);
        return false;
    }

    $command  = $CFG->netforum_dotdir;
    $command .= 'dot -Kfdp -Gcharset=utf8 -T' . escapeshellarg($fmt) . ' ' . escapeshellarg($dotfile);

    // dotmagiaa
    $data = `$command`;

    // siivous
    unlink($dotfile);

    if($return){
        return $data;
    } else {
        echo $data;
        return true;
    }
}

/**
 * Palauttaa yksittäisen ilmentymän uusimmat viestit
 *
 * @author Juho Kilpikoski
 * @param $netforumid ilmentymän id
 * @param $rows haettavien viestien määrän. Negatiivisista arvoista käytetään itseisarvoa.
 * @return array palautetaan taulukko, jonka alkiot ovat kyselyn tulosrivejä vastaavia olioita, tai false.
 *   Olioiden sisältämät tiedot ovat id, subject, userid, released, firstname, lastname ja
 *   adjacent (naapurien määrä). Virhetilanteessa palautetaan false.
 */
function netforum_get_newest($netforumid, $rows = 15) {
    global $CFG;
    $id = intval($netforumid);
    $sql = "
SELECT p.id, p.subject, p.userid, p.released, u.firstname, u.lastname
FROM {$CFG->prefix}netforum_post p, {$CFG->prefix}user u
WHERE p.netforumid = $id  
AND u.id = p.userid
AND p.released != 0

ORDER BY released DESC
LIMIT 0," . abs(intval($rows));

    return get_records_sql($sql);
}
/**
 * Palauttaa yksittäisen ilmentymän luetuimmat viestit
 *
 * @author Juho Kilpikoski
 * @param $netforumid ilmentymän id
 * @param $rows haettavien viestien määrän.  Negatiivisista arvoista käytetään itseisarvoa.
 * @return array palautetaan taulukko, jonka alkiot ovat kyselyn tulosrivejä vastaavia olioita, tai false.
 *   Olioiden sisältämät tiedot ovat id, subject, userid, released, firstname, lastname ja
 *   adjacent (naapurien määrä). Ei palauta viestejä joita ei ole luettu kertaakaan. Virhetilanteessa palutetaan false.
 */
function netforum_get_most_popular($netforumid, $rows = 15){
    global $CFG;
    $id = intval($netforumid);
    
    $sql = "
SELECT p.id, p.subject, p.released, SUM(r.times) AS times, p.userid
FROM {$CFG->prefix}netforum_post p, {$CFG->prefix}netforum_read r
WHERE p.netforumid = $id 
AND p.id = r.postid
AND p.released != 0

GROUP BY p.id
ORDER BY times DESC
LIMIT 00," . abs(intval($rows));
 
    return get_records_sql($sql);
}
/**
 * Tulostaa yksittäisen ilmentymän uusimmat viestit sisältävän lohkon
 * 
 * @author Juho Kilpikoski
 * @param $netforumid ilmentymän id
 * @return ei mitään
 */
function netforum_print_newest($netforumid){
    global $CFG, $USER;

    // Jos asetuksia ei ole asetettu ilmoitetaan tilanteesta
    if(!isset($CFG->netforum_newest_rows)){
        $html = '<p class="errorbox mod-netforum-center">' . get_string('settings_unset', 'netforum') .
                ' ('.get_string('newestrows', 'netforum').')</p>';
    } else {
        $posts = netforum_get_newest($netforumid, $CFG->netforum_newest_rows);
        if($posts === false){
            $html = '<p class="mod-netforum-center">' . get_string('no_messages', 'netforum') . '</p>';
        } else {
            $html = '<ol>';
            foreach($posts as $row){
                // Korostukset 
                $class = 'class="' . netforum_highlight($row->id, $row->userid) .'"';
                
                $id = netforum_get_cmid($netforumid);
                $html .= "<li><a href=\"post.php?id={$id}&amp;post=$row->id\" {$class}>$row->subject</a></li>";
            }
            $html .= "</ol>";
        }
    }
    $strnewest = get_string('newest', 'netforum');
    print_side_block($strnewest, $html);


}

/**
 * Tulostaa yksittäisen ilmentymän naapuririkkaimmat viestit sisältävän lohkon
 *
 * @author Juho Kilpikoski 
 * @param $netforumid ilmentymän id
 * @return ei mitään
 */
function netforum_print_most_popular($netforumid){
    global $CFG, $USER;
    
    // Jos asetuksia ei ole asetettu ilmoitetaan tilanteesta
    if(!isset($CFG->netforum_most_read_rows)){
        $html = '<p class="errorbox mod-netforum-center">' . get_string('settings_unset', 'netforum') .
                ' ('.get_string('mostreadrows', 'netforum').')</p>';
    } else {
        
        $posts = netforum_get_most_popular($netforumid, $CFG->netforum_most_read_rows);
        if($posts === false){
            $html = '<p class="mod-netforum-center">' . get_string('no_messages', 'netforum') . '</p>';
        } else {
            $html = '<ol>';
            foreach($posts as $row){
                // Korostukset 
                $class = 'class="' . netforum_highlight($row->id, $row->userid) .'"';
                
                $id = netforum_get_cmid($netforumid);
                $html .= "<li><a href=\"post.php?id={$id}&amp;post=$row->id\" {$class}>$row->subject</a></li>";
            }
            $html .= "</ol>";
        }
    }
    $strpoular = get_string('most_popular', 'netforum');

    print_side_block($strpoular, $html);

}
/**
 * Palauttaa tiedon montako kertaa käyttäjä on lukenut tietyn viestin.
 * 
 * @author Juho Kilpikoski
 * 
 * @param $userid käyttäjän id
 * @param $postid viestin id
 * @return int lukukertojen määrä, virhetilanteessa 0
 */
function netforum_has_read_post_times($userid, $postid){
     global $CFG;
    $temp = get_record("netforum_read", 'postid', intval($postid), 'userid' ,intval($userid) );
    if($temp){
        return $temp->times;
    } else {
        return 0;
    }
}

/**
 * Palauttaa netforum id:tä vastaavan cm-id:n
 * 
 * @author Juho Kilpikoski
 * @param $netforumid
 * @return int cmid tai false jos vastaava cm-idtä ei ole
 */
function netforum_get_cmid($netforumid){
    $cm = get_coursemodule_from_instance('netforum', $netforumid);
    if($cm){
        return $cm->id;
    }
    return false;

}
/**
 * Palauttaa postid:tä vastaavan netforumid:n
 * 
 * @author Juho Kilpikoski
 * @param $postid
 * @return int forumid tai false jos vastaavaa netforum-idtä ei ole
 */
function netforum_get_id($postid){
    global $CFG;
    
    $sql = "SELECT n.id FROM {$CFG->prefix}netforum n, {$CFG->prefix}netforum_post p WHERE n.id = p.netforumid AND p.id =" . intval($postid);
    $id = get_record_sql($sql);
    if($id){
        return $id->id;
    }
    return false;
}


/**
 * Hakee viestin tietokannasta
 *
 * @author Mikko Johansson
 * @param $postid viestin ID
 */
function netforum_get_post($postid) {
    global $CFG, $USER;

    return get_record_sql("SELECT p.*, u.firstname, u.lastname, u.email, u.picture, u.imagealt
                             FROM {$CFG->prefix}netforum_post p
                                  JOIN {$CFG->prefix}user u ON p.userid = u.id
                            WHERE p.id = $postid AND (p.released != 0 OR (p.released = 0 AND p.userid = {$USER->id}))");
} 

/**
 * Hakee käyttäjän lukutiedot tietokannasta
 *
 * @author Mikko Johansson
 * @param $postid viestin ID
 */
function netforum_get_read($postid) {
    global $CFG, $USER;

    return get_record_sql("SELECT r.*
                             FROM {$CFG->prefix}netforum_read r
                            WHERE r.userid = {$USER->id} AND r.postid = $postid");
} 

/**
 * Hakee käyttäjän lähdelistan tietokannasta
 *
 * @author Mikko Johansson
 * @param $postid viestin ID
 */
function netforum_get_notes($postid, $netforumid) {
    global $CFG, $USER;

    return get_records_sql("SELECT n.postid, p.subject
                             FROM {$CFG->prefix}netforum_note n, 
                                  {$CFG->prefix}netforum_post p
                            WHERE p.netforumid=$netforumid AND  
                                  n.postid=p.id AND
                                  n.userid = {$USER->id} AND 
                                  n.link = 1 AND 
                                  n.postid != $postid");
} 

/**
 * Hakee viestiin valitut linkit (verkon) tietokannasta
 *
 * @author Mikko Johansson
 * @param $postid viestin ID
 */
function netforum_get_links($postid) {
    global $CFG, $USER;

    return get_records_sql("SELECT w.parent
                             FROM {$CFG->prefix}netforum_web w
                            WHERE w.child = {$postid} AND w.value = 0");
} 


/**
 * Poistaa parametrissa olevat lähteet tietokannasta
 *
 * @author Mikko Johansson
 * @param $notelist viestissä käytetyt lähteet
 */
function netforum_delete_used_notes($notelist) {
    global $USER;
    foreach($notelist as $key => $value) {
        $noteid = netforum_get_noteid($key, $USER->id, 1);
        delete_records("netforum_note", 'id', $noteid);
    } 
}

/**
 * Näyttää viestin
 *
 * @author Mikko Johansson
 * @param object $post
 */
function netforum_post_view($netforum, $post, $course, $cm) {
    global $USER, $CFG;
    $context = get_context_instance(CONTEXT_MODULE, $cm->id);
    $canrate = (has_capability('mod/netforum:canrate',$context) AND $post->userid != $USER->id AND $netforum->assessed != 0);
    $scale = make_grades_menu($netforum->scale);
    $rating = new object();
    //jos saa arvioida printataan formin alku
    if(($netforum->assesstimestart > $post->released or $netforum->assesstimefinish < $post->released) AND ($netforum->assesstimestart != 0 and $netforum->assesstimefinish != 0)) {
        $canrate=false;
    }
    if($canrate) {
        echo '<form id="form" method="post" action="rate.php">';
        echo '<div class="ratingform">';
        echo '<input type="hidden" name="netforumid" value="'.$netforum->id.'" />';
    }

    //read-sql JOS EI oma viesti
    if($post->userid != $USER->id) {
        $read_post = new object();
        $read_post->lastread = time();
        $read_post->postid = $post->id;
        $read_post->userid = $USER->id;

        $tmp_read = netforum_get_read($post->id);
        if(!$tmp_read) {
        // JOS EI OLE AIEMMIN lukenut
            $read_post->firstread = time();
            $read_post->times = 1;
           insert_record("netforum_read", $read_post);
        }
        else {
        //JOS ON
            $tmp_read->times = $tmp_read->times + 1;
            $tmp_read->lastread = time();
            update_record("netforum_read", $tmp_read);
        }
    }
    // alotetaan taulukko
    echo '<table cellspacing="0" class="forumpost mod-netforum-fullwidth">';

    //käyttäjän kuva
    $postuser = new object();
    $postuser->id        = $post->userid;
    $postuser->firstname = $post->firstname;
    $postuser->lastname  = $post->lastname;
    $postuser->imagealt  = $post->imagealt;
    $postuser->picture   = $post->picture;

    echo '<tr class="header"><td class="picture left">';
    print_user_picture($postuser, $course->id);
    echo '</td>';

    //otsikko
    echo '<td class="topic starter">';
    if (!empty($post->subjectnoformat)) {
        echo '<div class="subject">'.$post->subject.'</div>';
    } 
    else {
        echo '<div class="subject">'.format_string($post->subject).'</div>';
    }

    //käyttäjä
    echo '<div class="author">';
    $fullname = fullname($postuser);
    $by = new object();
    $tmpstrreleased = userdate($post->released);
    $by->date = $tmpstrreleased;
    if($post->released == 0){
        $tmpstrreleased = get_string('thisisdraft', 'netforum');
    }
    
    
    
    echo '<a href="'.$CFG->wwwroot.'/user/view.php?id='.$post->userid.
            '&amp;course='.$course->id.'">'.$fullname.'</a> - '.$tmpstrreleased;
    echo '</div></td></tr>';

    //group läppä?
    echo '<tr><td class="left side">&nbsp;</td><td class="content">'."\n";

    $options = new object();
    $options->para      = false;
    $options->trusttext = true;
    
    // viesti
    echo '<div class="posting">';
    echo format_text($post->message, $post->format, $options, $course->id);
    echo '</div>';
    
    //ratings
    
    echo '<div class="ratings">';
    if ($USER->id == $post->userid) {
        if ($grades = netforum_get_user_grades($netforum, $post->userid)) {
            netforum_print_ratings($post->id, $scale);
        }
        
    } else if ($canrate) {
        if (!isset($myrating)) {
        //if (is_null($myrating)) {
        if ($rating = get_record("netforum_ratings", "userid", $USER->id, "postid", $post->id)) { 
              $myrating = $rating->rating;
          } else {         
              $myrating = NETFORUM_UNSET_POST_RATING; 
              $rating->postid = $post->id;
          }
      }
        
        if ($grades = netforum_get_user_grades($netforum, $post->userid)) {
            netforum_print_ratings($post->id, $scale);
        }
        netforum_print_rating_menu($post->id, $USER->id, $scale, $myrating);
        
    }
    if ($grades = netforum_get_user_grades($netforum, $post->userid) AND !$canrate AND has_capability('mod/netforum:canrate',$context)) {
        netforum_print_ratings($post->id, $scale);
    }
    if ($canrate) {
        //echo '<div class="ratingsubmit">';
        echo '<input type="submit" value="'.get_string('sendinratings', 'netforum').'" />';
        if ($netforum->scale < 0) {
            if ($scale = get_record("scale", "id", abs($netforum->scale))) {
                print_scale_menu_helpbutton($course->id, $scale );
            }
        }
        //echo '</div>';
        echo '</div>';
        echo '</form>';
    }
    echo '</div>';
    
    
    // lopetetaan taulukko
    echo '</td></tr></table>'."\n\n";
    // Lopetetaan ratingform
   
                            
}
/**
 *Tulostaa arviointi pudotusvalikon
 *
 *@author Tevje Olin
 *@param $postid, $userid, $scale, $myrating
 *@return nothing
 */
function netforum_print_rating_menu($postid, $userid, $scale, $myrating=NULL) {
  
      static $strrate;
  
      if (is_null($myrating)) {
          if (!$rating = get_record("netforum_ratings", "userid", $userid, "post", $postid)) {
              $myrating = NETFORUM_UNSET_POST_RATING;
          } else {
              $myrating = $rating->rating;
          }
      }
  
      if (empty($strrate)) {
          $strrate = get_string("notrated", "netforum");
      }
      $scale = array(NETFORUM_UNSET_POST_RATING => $strrate) + $scale;
      choose_from_menu($scale, $postid, $myrating, '');
}
/**
 *Tulostaa arvioinnit
 *
 *@author Tevje Olin
 *@param $postid, $scale, $ratings
 *@return nothing
 */

function netforum_print_ratings($postid, $scale, $ratings=null) {
  
    $strratings = '';
    $agg        = netforum_get_ratings_mean($postid, $scale, $ratings);
    $strratings = get_string("aggregateavg", "netforum");
    if ($agg !== "") {
        if (empty($strratings)) {
            $strratings = get_string("ratings", "netforum");
        }  
        echo "$strratings: ";
        echo "$agg ";
        }
      
  }
/**
 *Hakee viestin arvioinnit ja laskee niiden keskiarvon
 *
 *@author Tevje Olin
 *@param $postid, $scale, $ratings
 *@return string
 */
function netforum_get_ratings_mean($postid, $scale, $ratings=NULL) {
  
      if (is_null($ratings)) {
          $ratings = array();
          if ($rates = get_records("netforum_ratings", "postid", $postid)) {
              foreach ($rates as $rate) {
                  $ratings[] = $rate->rating;
              }
          }
      }
  
      $count = count($ratings);
  
      if ($count == 0 ) {
          return "";
  
      } else if ($count == 1) {
          $rating = reset($ratings);
          return $scale[$rating];
  
      } else {
          $total = 0;
          foreach ($ratings as $rating) {
              $total += $rating;
          }
          $mean = round( ((float)$total/(float)$count) + 0.001);  // varmistetaan että 0,5 pyöristyy ylös
  
          if (isset($scale[$mean])) {
              return $scale[$mean]." ($count)";
          } else {
              return "$mean ($count)";    
          }
      }
}




/**
 * Lisää uuden viestin
 *
 * @author Mikko Johansson
 * @param object $post
 * @return int
 */
function netforum_post_add($post, $notelist) {
    global $USER;
    
    $post->created      = $post->modified = time();
    $post->modifiersid  = $USER->id;

    //ei vielä koodattu mitenkään
    $post->format = 1;
    $post->attachment   = "";
    
    if($post->released)
        $post->released = time();
    //seivataan viesti
    if(!$post->id = insert_record("netforum_post", $post))
        return 0;
        
    //jos julkaistiin
    if($post->released) {
        //kirjoitetaan webiin
        foreach($notelist as $key => $value) {
            $tmp_web = new object();
            $tmp_web->parent = $key;
            $tmp_web->child = $post->id;
            $tmp_web->value = $value;
            insert_record("netforum_web", $tmp_web);
            
            netforum_delete_used_notes($notelist);
        } 
    }
    else {
        //poistetaan aiemmat webit
        delete_records("netforum_web", 'child', $post->id);
        //kirjoitetaan webiin
        foreach($notelist as $key => $value) {
            $tmp_web = new object();
            $tmp_web->parent = $key;
            $tmp_web->child = $post->id;
            $tmp_web->value = 0;
            insert_record("netforum_web", $tmp_web);
        }
    }
    
    return $post->id;
}

/**
 * Päivittää viestin
 *
 * @author Mikko Johansson
 * @param object $post netforumin post-object
 * @return bool
 */
function netforum_post_update($post, $notelist) {
    global $USER, $CFG;

    $post->modified = time();
    if($post->released)
        $post->released = time(); 
    //päivitetään postaus
    update_record('netforum_post', $post);
    
    if($post->released) {
        //poistetaan aiemmat webit
        delete_records("netforum_web", 'child', $post->id);
        //kirjoitetaan webiin
        foreach($notelist as $key => $value) {
            $tmp_web = new object();
            $tmp_web->parent = $key;
            $tmp_web->child = $post->id;
            $tmp_web->value = 1;
            insert_record("netforum_web", $tmp_web);
        }
    }
    else {
        //poistetaan aiemmat webit
        delete_records("netforum_web", 'child', $post->id);
        //kirjoitetaan webiin
        foreach($notelist as $key => $value) {
            $tmp_web = new object();
            $tmp_web->parent = $key;
            $tmp_web->child = $post->id;
            $tmp_web->value = 0;
            insert_record("netforum_web", $tmp_web);
        }
    }
    
    return true;
}


/**
 * Poistaa viestin sisällön
 *
 * @author Mikko Johansson
 * @param object $post netforumin post-object
 * @return bool
 */
function netforum_post_clear($post) {
    global $USER;

    $post->message = get_string('deleted','netforum');
    $post->modified = time();
    $post->modifiersid = $USER->id;
    
    if (update_record("netforum_post", $post)) {
        return true;
    }
    return false;
}



/**
 * Poistaa luonnoksen
 *
 * @author Mikko Johansson
 * @param object $post netforumin post object (käytetään vain id)
 * @return bool
 */
function netforum_post_draft_delete($postid) {
    global $USER, $CFG;

    $post = get_record_sql("SELECT p.*
                             FROM {$CFG->prefix}netforum_post p
                            WHERE p.id = $postid AND p.released = 0 AND p.userid = {$USER->id}");

    //poistettiinko postaus
    if ($post) {
        return netforum_post_delete($postid);
    }
    
    return false;
}

/**
 * Poistaa viestin
 *
 * @author Mikko Johansson
 * @param object $post netforumin post object (käytetään vain id)
 * @return bool
 */
function netforum_post_delete($postid) {

    //poistettiinko postaus
    if (delete_records("netforum_post", 'id', $postid)) {
    
      //delete ratings
      delete_records("netforum_ratings", 'postid', $postid);
      //delete read info
      delete_records("netforum_read", 'postid', $postid);
      //delete notes
      delete_records("netforum_note", 'postid', $postid);
      //delete web
      delete_records("netforum_web", 'parent', $postid);
      //delete web
      delete_records("netforum_web", 'child', $postid);
      
      return true;
    }
    
    return false;
}

function netforum_get_noteid($postID, $userID, $link=0){
    global $CFG;
    
    $sql = "SELECT id FROM {$CFG->prefix}netforum_note WHERE postid=".$postID." and userid=".$userID." AND link=".$link." ";
    
    if( !($noteID = get_record_sql($sql)))
    {
//        error("netforum_get_noteid ei onnistunut");
        return false;
    }
    //error("ei onnistunut");
    
    return $noteID->id;
}

/**
 *  @author: Aleksi Toivanen - Ohtuprojektiryhmä modeX
 *  funktio: Funktio lisää note-tauluun uuden rivin.
 *  syötteet:
 *  @param int    $postID  sen viestin id, johon ollaan lisäämässä muistiinpanoa
 *  @param int    $userID  käyttäjän id
 *  @param string $note    viestin sisältö stringinä
 *  @param int    $link    tieto siitä lisätäänkö postaus linkkilistaan 
 *                (1, jos toimii linkkinä; 0, jos ei toimi linkkinä)
 *  @return boolean true, jos onnistui
 */
function netforum_note_add($postID, $userID, $note, $link=0)
{
    
    $syote = new object;
    $syote->postid = $postID;
    $syote->userid = $userID;
    $syote->link = 0;
    $syote->comment = $note;
    
    
    if( !insert_record("netforum_note", $syote) )
    {
        //insert ei onnistunutkaan, heitetään error
        error("muistiinpanon lisääminen ei onnistunut");
    }
    
    return true;
}

/**
 *  @author: Aleksi Toivanen - Ohtuprojektiryhmä modeX
 *  funktio: Funktio tekee annetun id:n perusteella note-tauluun 
 *           parametrina annetun muutoksen kenttään comment.
 *  syötteet:   
 * @param int    $noteID  sen viestin id, johon ollaan lisäämässä muistiinpanoa
 * @param string $note    viestin sisältö stringinä
 * @param int    $link    tieto siitä lisätäänkö postaus linkkilistaan 
 *                        (1, jos toimii linkkinä; 0, jos ei toimi linkkinä)
 * @return boolean  true, jos onnistui
 */
function netforum_note_edit($noteID, $note, $link=0)
{

    //objectina
    $paivitys = new object;
    $paivitys->id = $noteID;
    $paivitys->comment = $note;
    
    //päivitä tieto kantaan, update_record saa toisena parametrinaan data-objectin..
    if( !update_record('netforum_note', $paivitys) )
    {
        //jos päivitys ei onnistunut, ilmoitetaan siitä errorin muodossa
        error("Päivitys ei onnistunut");
    }
    return true;
}

/**
 *  @author: Aleksi Toivanen - Ohtuprojektiryhmä modeX
 *  funktio: Apufunktio joka hakee kaikki tiettyyn viestin liittyvät annetun käyttäjän näkemät kommentit 
 *  syöte:
 *  @param int $netforumID    netforumin id
 *  @param int $postID        viestin id
 *  @param int $userID        käyttäjän id, jolle näkyvissä olevat kommentit haetaan
 *  @param ??  $coursecontext moodlen kurssikonteksti
 *  @return mixed  array, jossa notet, jos notea löytyy; false, jos noteja ei löydy
 */
function netforum_get_visible_notes($netforumID, $postID, $userID, $coursecontext){

    //muuttujat:
    global $CFG;
    global $USER;
    
//    $sql = "SELECT id FROM {$CFG->prefix}netforum_note WHERE postid={$postID} AND link=0";
    $sql = "netforum_note WHERE postid={$postID} AND link=0";
    
    //tarkistetaan löytyykö viestille ollenkaan kommentteja
    if(record_exists($sql)){
        //löytyi kommentteja, joten ruvetaan setvimään niistä mielenkiintoisia..
        
        $kaikkiNotet = netforum_get_notes_for_post($postID);
        $nakyvatNotet = false;

        //Erottele turhat notet joukosta
        foreach($kaikkiNotet as $note){
        
            //tarkista onko noten kirjoittaja ope
            //tarkista onko noten kirjoittanut $USER        
            //tarkista onko noten kirjoittanut alkuperäisen viestin kirjoittanut kaveri
            //eli:
            //tarkistetaan onko kyseinen note tälle käyttäjälle näkyvä
            //TODO: pitäiskö viestissä näkyvillä kommenteilla olla joku järjestys?? jos pitäisi, niin se olisi helpointa toteuttaa tässä. Paitsi että ehkä sen voi tehdä jotenkin kivasti muualla css:llä
            if(netforum_check_note_visibility($note->noteID,
                                              $note->notenTekijanID, 
                                              $note->viestinKirjoittajanID, 
                                              $USER->id,
                                              $coursecontext)){
                //lisätään note kiinnostavien (näkyvien) notejen listaan
                $nakyvatNotet[] = $note;
            }
            //näkymättömille noteille ei tehdä mitään
        
        }
    
        return $nakyvatNotet;
    
    }
    
    else{ //jos noteja ei ollut
        return false;
    }
}

/**
 *  @author: Aleksi Toivanen - Ohtuprojektiryhmä modeX
 *  funktio: apufunktio, joka hakee kaikki sille parametrina annetun viestin kommentit.
 *  syöte:
 *  @param int $postID      viestin id, jonka kommentit halutaan hakea
 *  @return mixed           array, jos noteja löytyy; false, jos ei
 */
function netforum_get_notes_for_post($postID){
    //muuttujat:
    global $CFG;
    
    $palaute;
    $palaute1;
    $palaute2;
    
    //luodaan kysely &
    $sql = "SELECT id, postid, userid, comment FROM {$CFG->prefix}netforum_note WHERE postid={$postID} AND link=0 ";
    $sql2 = "SELECT userid FROM {$CFG->prefix}netforum_post WHERE id={$postID} ";
    
//    echo $sql."<br />";
//    echo $sql2;
    
    //haetaan kannasta
    if( !($palaute1 = get_records_sql($sql, 0, 1000)))
    {
        //jos ei onnistuttu haussa, niin ilmoitetaan siitä errorilla
        error("get_records_sql ei onnistunut ...");
        return false;
    }
    
    //haetaan kannasta
    if( !($palaute2 = get_record_sql($sql2)))
    {
        //jos ei onnistuttu haussa, niin ilmoitetaan siitä errorilla
        error("get_record_sql ei onnistunut ...");
        return false;
    }

    //siirretään kaikki arrayssa saadut objectit toiseen paremmin nimettyyn array:hyn talteen
    $i=0;
    foreach($palaute1 as $palaute1rivi){
        $palaute[$i]->noteID = $palaute1rivi->id;
        $palaute[$i]->notenTekijanID = $palaute1rivi->userid;
        $palaute[$i]->viestinKirjoittajanID = $palaute2->userid;
        $palaute[$i]->viesti = $palaute1rivi->comment;
        $palaute[$i]->postID = $palaute1rivi->postid;
        $i++;
    }
    
    //palautetaan 
    return $palaute;
}

/**
 *  @author: Aleksi Toivanen - Ohtuprojektiryhmä modeX
 *  funktio: Funktio tulostaa kaikki käyttäjälle näkyvät parametrina annetun viestin kommentit
 *  syöte:
 *  @param int $netforumID  netforumin id
 *  @param int $postID      viestin id, jonka kommentit tulostetaan
 *  @param int $userID      käyttäjän id, jolle näkyvissä olevat kommentit tulostetaan
 *  @param int $course      TODO?? mikäs tän kuvaus vois olla??
 *  palauttaa: ei palauta mitään
 */
function netforum_note_view($netforumID, $postID, $userID, $course)
{
    $coursecontext = get_context_instance(CONTEXT_COURSE, $course->id);
    //tarkista löytyykö postille viestejä, jos löytyy, niin hae ja tulosta ne
    if($kaikkiNotet = netforum_get_visible_notes($netforumID, $postID, $userID, $coursecontext)){

        foreach($kaikkiNotet as $note){
            netforum_note_show($netforumID, $note->postID, $note->notenTekijanID);
        }
    
    }

}

/**
 *  @author: Aleksi Toivanen - Ohtuprojektiryhmä modeX
 *  funktio: Apufunktio, joka tulostaa annetun viestin kommentin annetulta käyttäjältä
 *  syöte:
 *  @param int $netforumID  netforumin id
 *  @param int $postID      viestin id, jonka kommenttia ollaan tulostamassa
 *  @param int $noteMakerID käyttäjän id, jonka kommentti viestistä tulostetaan
 *  palauttaa: ei palauta mitään
 */
function netforum_note_show($netforumID, $postID, $noteMakerID)
{
    //globaalit muuttujat
    global $CFG;
    global $USER;
    
    //muuttujat:
    $viesti;
    $muistiinpano;
    $noteOlemassa = true; //oletetaan että annetulle postille on jo note olemassa
    
    $oma = false;
    $opettajan = false;
    $lisays = false;    
    
    //käännökset
//    $str = get_string('', 'netforum');


    //tarvitaan $course myöhempää käyttöä varten
    if ($netforumID) {
        /// Haetaan coursemodule id:n perusteella
        if (! $cm = get_coursemodule_from_id('netforum', $netforumID)) {
            error('Course Module ID was incorrect');
        }
        /// coursemodulista haetaan course-olio
        if (! $course = get_record('course', 'id', $cm->course)) {
            error('Course is misconfigured');
        }
    }
    
    //tarkastetaan onko note oleassa
    //if( !$noteID = netforum_get_noteid($postID, $USER->id)){
    if( !$noteID = netforum_get_noteid($postID, $noteMakerID)){
        
        $noteOlemassa = false;
//        $viesti = "";
//        $linkki = "";
    
    }
    
    
    
    //jos note oli olemassa eikä ollut mitään parametrejä, niin haetaan noten tiedot vain kannasta
    else{
    
//        $muistiinpano = netforum_note_get($noteID->id, $USER->id);
        $muistiinpano = netforum_note_get($noteID, $USER->id);
        $viesti = $muistiinpano->comment;
    
    }
    
    //selvitä kurssikonteksti -- tarvitaan has_capability-funktiota varten
    $coursecontext = get_context_instance(CONTEXT_SYSTEM, $course->id );

    //tarkista onko opettaja.. 
    //jotain alla olevan suuntaista
    //if (has_capability('moodle/legacy:teacher', $sitecontext, NULL, false)) {
        //näytä rate_ominaisuudet
        //netforum_rate_view($rateID);
    //}
    

    if($noteOlemassa){
        
        //selvitetään kuka on tehnyt viestin
        $sql = "SELECT userid FROM {$CFG->prefix}netforum_post WHERE id={$postID} ";       
        
        if( !($palaute = get_record_sql($sql)))
            {
                //jos ei onnistuttu haussa, niin ilmoitetaan siitä errorilla
                error("get_record_sql ei onnistunut ...");
                return false;
        } 
        
        //$notensuhde kuvaa noten kirjoittajan roolia: (nykyinen käyttäjä, viestin kirjoittaja, opettaja)
        $notensuhde = netforum_check_note_maker($noteID, $noteMakerID, $USER->id, $palaute->userid, $coursecontext);
        
        if($notensuhde == 1 | $notensuhde == 3 | $notensuhde == 5 | $notensuhde == 7){
            $oma=true;
        }
        if($notensuhde == 2 | $notensuhde == 3 | $notensuhde == 6 | $notensuhde == 7){
            $lisays=true;            
        }
        if($notensuhde == 4 | $notensuhde == 5 | $notensuhde == 6 | $notensuhde == 7){
            $opettajan=true;
        }

        
        //sijoitetaan taulukkoon
        echo '<table cellspacing="0" class="forumpost mod-netforum-fullwidth">';
        
        //haetaan tietoa noten tekijästä, tulostettavaksi noten yhteyteen
        $noteMakerInfo = netforum_get_user($noteMakerID);

        $noteUser = new object();
        $noteUser->id        = $noteMakerInfo->id;
        $noteUser->firstname = $noteMakerInfo->firstname;
        $noteUser->lastname  = $noteMakerInfo->lastname;
        $noteUser->imagealt  = $noteMakerInfo->imagealt;
        $noteUser->picture   = $noteMakerInfo->picture;

        //tulostetaan noten kirjoittajan kuva
        echo '<tr class="header"><td class="picture left">';
        print_user_picture($noteUser, $course->id);
        echo '</td>';
        
        
        //viestiin verrattuna otsikon tilalla muistiinpanossa lukee 
        //muistiinpano/opettajankommentti/lisäys(jos alk.per.viestin kirjoittajan kommentti)
        echo '<td class="topic starter">';
        
        //apumuuttujat tekstiä varten
        $strtekija = "";
        $strlisa = "";
            
        //TODO: NÄILLE JOKAISELLE VAIHTOEHDOLLE JOTAIN EROA STYLEIHIN.. tai ei oo pakko oikeastaan enää
        if($oma){//jos käyttäjän oma muistiinpano
            $strtekija = get_string('usersownnote','netforum');
            $strlisa = ", ";
        }
        if($lisays){//jos viestinkirjoittajan muistiinpano
            $strtekija = $strtekija.$strlisa.get_string('writersnote','netforum');
            $strlisa = ", ";
        }
        if($opettajan){//jos opettajan muistiinpano
            $strtekija = $strtekija.$strlisa.get_string('teacherscomment','netforum');
            $strlisa = ", ";
        }
        
        echo '<div class="subject">'.$strtekija;
        helpbutton('notetitle', get_string('notetitle', 'netforum'), 'netforum');
        echo '</div>';
        
        //käyttäjä
        echo '<div class="author">';
        //tähän muistiinpanon tekijän nimi
        $fullname = fullname($noteUser);
        echo $fullname;
        echo '</div></td></tr>';
        
        //aloitetaan toka rivi
        echo '<tr>';
        
        //??
        echo '<td></td>';
        
        //tulostetaan muistiinpanon varsinainen tekstiosuus
        echo '<td>'.$viesti.'</td>';
        
        

        
        echo '</tr>';
        
        //lopetetaan taulukko
        echo '</table>';
    //formi jossa muistiinpano voidaan poistaa/muokata
        if($oma){
            echo '<div class="mod-netforum-center">';
            echo '<form action="'.$CFG->wwwroot.'/mod/netforum/note.php?netforumid='.$netforumID.'&noteid='.$noteID.'" method="post">';
            echo '<input type="submit" name="edit" value="'.get_string("edit","netforum").'"/>';
            echo '<input type="submit" name="delete" value="'.get_string("delete","netforum").'"/>';
            echo '</form>';
            echo '</div>';
        }
        
    }
    
}

/**
 *  @author: Aleksi Toivanen - Ohtuprojektiryhmä modeX
 *  funktio: Apufunktio, joka hakee kannasta kentät: id, firstname, lastname, email, picture ja imagealt
 *           user-taulusta.
 *  syöte:
 *  @param int $userID
 *  palauttaa:
 *  @return array jossa id, firstname, lastname, email, picture ja imagealt
 */
function netforum_get_user($userID){
    global $CFG;
    $sql = "SELECT id, firstname, lastname, email, picture, imagealt
            FROM {$CFG->prefix}user
            WHERE id = {$userID}";
            
    return get_record_sql($sql);
}

/**
 *  @author: Aleksi Toivanen - Ohtuprojektiryhmä modeX
 *  funktio: Apufunktio, joka hakee note-taulusta muistiinpanoon liittyvän viestin id:n.
 *  syöte:
 *  @param int $noteID
 *  palauttaa:
 *  @return int viestin id
 */
function netforum_get_postid_from_note($noteID){
    global $CFG;
    
    $sql = "SELECT postid FROM {$CFG->prefix}netforum_note WHERE id=".$noteID;
    
    if( !($postID = get_record_sql($sql)))
    {
//        error("netforum_get_noteid ei onnistunut");
        return false;
    }
    //error("ei onnistunut");
    
    return $postID->postid; 
}


/**
 *  @author: Aleksi Toivanen - Ohtuprojektiryhmä modeX
 *  funktio: Apufunktio, joka hakee note-taulusta comment- ja link-kentät. 
 *  syöte:
 *  @param int $noteID  haettavan viestin id
 *  @param int $userID  hakijan id (oikeastaan turha) TODO:poista tämä parametri.
 *  @param int $link    tieto siitä onko kyseessä linkki
 *  palauttaa: arrayn, jossa noten sisältö ja tieto siitä onko note linkki
 *          $palaute->muistiinpano  - muistiinpanon tekstisisältö
 *          $palaute->linkki        - tieto siitä onko muistiinpano samalla linkki linkkilistaa varten
 *  @return array
 */
function netforum_note_get($noteID, $userID, $link=0)
{
    //muuttujat:
    global $CFG;
    
    //luodaan kysely &
//    $sql = "SELECT comment, link FROM {$CFG->prefix}netforum_note WHERE id=".$noteID." AND userid=".$userID." AND link=".$link." ";
    $sql = "SELECT comment, link FROM {$CFG->prefix}netforum_note WHERE id=".$noteID." AND link=".$link." ";
    
    //haetaan kannasta
    if( !($palaute = get_record_sql($sql)))
    {
        //jos ei onnistuttu haussa, niin ilmoitetaan siitä errorilla
        error("get_record_sql ei onnistunut ...");
        return false;
    }
    
    //palautetaan 
    return $palaute;
}

/**
 *  @author: Aleksi Toivanen - Ohtuprojektiryhmä modeX
 *  funktio: Funktio poistaa parametrina annettua id:tä vastaavan rivin note-taulusta.
 *  syöte:
 *  @param int $noteID  poistettavan muistiinpanon id
 *  palauttaa:  
 *  @return boolean   true, jos onnistui.
 */
function netforum_note_delete($noteID)
{
    //poista muistiinpanon sisältö
    //vai pitäiskö tän vain tyhjentää sisältö ja jättää merkintä sisältö poistettu
    //toisaalta muistiinpano näkyy vain muistiinpanon kirjoittajalle, joten kai sen voisi poistaa
    if( !delete_records("netforum_note", "id", $noteID) )
    {
        //jos poisto epäonnistui, ilmoitetaan siitä errorilla
        error("Muistiinpanon poisto epäonnistui id:llä ".$noteID.".");
    }
    return true;
    
}

/**
 *  @author: Aleksi Toivanen - Ohtuprojektiryhmä modeX
 *  funktio: Apufunktio, joka palauttaa tiedon siitä näkyykö annettu viesti annetulle käyttäjälle.
 *  syöte:      
 *  @param int $noteID                  muistiinpanon id
 *  @param int $viestinKirjoittajanID   alkuperäisen viestin kirjoittajan id
 *  @param int $kayttajanUserID         kayttajan id, jonka näkevyyttä tarkistetaan
 *  @param ??  $coursecontext           moodlen kurssikonteksti
 *  palauttaa:
 *  @return boolean   tieto siitä näkyykö kommentti käyttäjälle.
 *              false, jos kommentti on sellainen, jota ei tarvitse näyttää
 *              true, jos kommentti on oma
 *              true, jos kommentti on viestin kirjoittajan
 *              true, jos opettajan kommentti
 */
function netforum_check_note_visibility($noteID, 
                                        $notenTekijanID, 
                                        $viestinKirjoittajanID, 
                                        $kayttajanUserID, 
                                        $coursecontext){
    
    //tarkista onko oma kommentti  
    if($notenTekijanID == $kayttajanUserID){
        return true;
    }
    
    //tarkista onko notentekija kommentin kirjoittaja
    else if($viestinKirjoittajanID == $notenTekijanID){
        return true;
    }
    
    //tarkista onko notentekijä ope
    //TODO: selvitä onko kommentin kirjoittaja opettaja - miten tehdään??
    else if (has_capability('mod/netforum:writeteachercomments', $coursecontext, $notenTekijanID, false)){
        return true;
    }
    
    else{//jos kommentin kirjoittaja on joku muu, eli se ei siis näy annetulle käyttäjälle
        return false;
    }
}

/**
 *  @author: Aleksi Toivanen - Ohtuprojektiryhmä modeX
 *  funktio:    Apufunktio tarkistaa onko kommentin kirjoittaja käyttäjä,viestin kirjoittaja vai opettaja.
 *              Funktio olettaa kaikkien syötteenä tulevien olevan näkyviä kommentteja, 
 *              eli toisinsanoen niiden tekijän tulisi kuulua johonkin ylläolevista ryhmistä.
 *  syöte:
 *  @param int $noteID - muistiinpanon id
 *  @param int $notenTekijanID - kommentin kirjoittajan id
 *  @param int $kayttajanUserID - käyttäjän id 
 *                  (käyttäjän id:tä ei oikeastaan tarvitsisi siirtää parametrina, 
 *                  koska se löytyy myös $USER-globaalista muuttujasta)
 *  
 *  @return int luku välillä 1-3, joka määrittelee oliko muistiinpano kenen kirjoittama.
 *  palauttaa:  1, jos kommentti on oma.
 *              2, jos kommentti on viestin kirjoittajan.
 *              3, jos opettajan kommentti, pitiks tää opettajan kommentti näkyä nyt sit kaikille??
 */
function netforum_check_note_maker( $noteID, 
                                    $notenTekijanID, 
                                    $kayttajanUserID, 
                                    $viestinKirjoittajanID, 
                                    $coursecontext){

    $palaute = 0;
    
    //tarkista onko oma kommentti  
    if($notenTekijanID == $kayttajanUserID){
        $palaute = 1;
    }
                                    
    
    //muutoin tekijän täytyy olla myös viestin kirjoittaja
    //tarkista onko kommentin tekija myös viestin kirjoittaja
    if($viestinKirjoittajanID == $notenTekijanID){
        $palaute = $palaute + 2;
    }    

    //tarkista onko notentekijä ope
    if(has_capability('mod/netforum:writeteachercomments', $coursecontext, $notenTekijanID, false)){
        $palaute = $palaute + 4;
    }

    
    return $palaute;
    
}

/**
 * Laskee yksittäisne viestin vanhempien määrän
 *
 * @author Juho Kilpikoski
 *
 * @param $id viesti jonka vanhempien määrää lasketaan
 * @return int vanhempien määrä
 */

function netforum_count_parents($id){
    return count_records("netforum_web", "child", intval($id));
}
/**
 * Hakee joko kaikki yhteen netforumilmentymään liittyvät kirjoitukset tai yhden kirjoittajan yhteen
 * netforumiin liittyvät kirjoitukset.
 *
 * @author Juho Kilpikoski
 * @param $forumid haettavan forumilmentymän id
 * @param $userid false jos kaikki, muutoin userid-int
 * @param $order järjestys jossa viestilista palautetaan sql-kyselyn kokonaisena order-osana
 * @return array palautetaan taulukko, jonka alkiot ovat kyselyn tulosrivejä vastaavia olioita, tai false.
 *   Olioiden sisältämät tiedot ovat id, subject, userid, released, firstname, lastname ja
 *   children (lasten määrä). false palautetaan jos kysely jostakin syystä epäonnistuu.
 *
 */
function netforum_get_posts_for_list($netforumid, $userid = false, $order = 'ORDER BY released ASC') {
    global $CFG;


    $id = intval($netforumid);

    // Jos käyttäkohtainen listaus niin lisätään ehto käyttäjän valitsemiseksi
    if($userid === false){
        $userclause = '';
    } else {
        $userclause = "AND u.id = " . intval($userid);
    }


    $sql = "(
    SELECT p.id, p.subject, p.userid, p.released, u.firstname, u.lastname, COUNT( w.child ) AS children
FROM {$CFG->prefix}netforum_post p, {$CFG->prefix}netforum_web w, {$CFG->prefix}user u
WHERE p.netforumid = $id 
AND u.id = p.userid
AND p.id = w.parent
AND p.released != 0
    $userclause
GROUP BY p.id

)
UNION (

SELECT p.id, p.subject, p.userid, p.released, u.firstname, u.lastname, 0 AS children
FROM {$CFG->prefix}netforum_post p, {$CFG->prefix}user u
WHERE p.netforumid = $id
AND u.id = p.userid
AND p.id NOT 
IN (
SELECT parent
FROM {$CFG->prefix}netforum_web
)
AND p.released != 0
    $userclause
)
    $order";


    return get_records_sql($sql);
}

function netforum_get_view_actions() {
    return array('view help', 'view tagcloud', 'view list', 'view citations',
                 'view notes', 'view drafts', 'view statistics', 'view neighbourgraph',
                 'view all', 'view post');
}

function netforum_get_post_actions() {
    return array('delete citation', 'delete draft', 'delete note', 'add note',
                 'update note', 'add citation', 'update post', 'add post',
                 'delete post', 'update rating', 'add rating');
}

?>