<?php

// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.


/**
 * Tulostaa yksittäisen netforumilmentymän kaikista viesteistä listauksen
 * 
 * Perinteinen listausnäkymä joka mahdollistaa viestien järjestämisen
 * kaikkien sarakkeidensa perusteella. Näin käyttäjä löytää sellaisetkin
 * viestit joiden löytäminen otsikkopilvestä ei ole niin helppoa. Lisäksi
 * mahdollistaa yksittäisen käyttäjän viestien listaamisen jotta käyttäjä
 * voi seurata toisia käyttäjiä joiden tekstit hän on havainnut 
 * mielenkiintoisiksi. 
 *
 * @package   mod-netforum
 * @copyright 2009 Helsingin Yliopisto, Tietojenkäsittelytieteen laitos
 * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

global $USER;


require_once ("lib.php");
require_once ($CFG->libdir.'/tablelib.php');

// valinnaiset parametrit yksittäisen käyttäjän viesitlistaukseen ja järjestykseen
$userid = optional_param('userid', false, PARAM_INT);


// Sivun url + perusparametrit
$currenturl =  "view.php?id={$id}&amp;tab=" . NETFORUM_TAB2 .  "";
$currenturlall =  "view.php?id={$id}&amp;tab=" . NETFORUM_TAB2 .  "";
if($userid){
    $currenturl .= "&amp;userid={$userid}";
}



// Lokalisaatiot
$strsubject  = get_string('subject', 'netforum');
$strauthor  = get_string('author', 'netforum');
$strreleased  = get_string('release_time', 'netforum');
$stradjacent  = get_string('adjacent', 'netforum');
$strparents  = get_string('numberofparents', 'netforum');
$strchildren  = get_string('numberofchildren', 'netforum');
$strlist  = get_string('postlist', 'netforum');
$strshowall  = get_string('show_all_posts', 'netforum');
$strreads  = get_string('timesread', 'netforum');
$strgrade  = get_string('grade', 'netforum');
$strnorate = get_string('notrated', 'netforum');

$canviewgrade = true;

// Luodaan ja alustetaan pääsisällön muodostava flexibletable
if(!isguestuser()){
    $tablecolumns = array( 'subject', 'author', 'released', 'children', 'parents', 'reads', 'grade');
    $tableheaders = array( $strsubject, $strauthor, $strreleased, $strchildren, $strparents, $strreads, $strgrade);
} else {
    $tablecolumns = array( 'subject', 'author', 'released', 'children', 'parents', 'reads');
    $tableheaders = array( $strsubject, $strauthor, $strreleased, $strchildren, $strparents, $strreads);
}
$table = new flexible_table('mod-netforum-list-posts');

$table->define_columns($tablecolumns);
$table->define_headers($tableheaders);
$table->define_baseurl($CFG->wwwroot.'/mod/netforum/'.$currenturl);

// Oletusjärjestys: uusimmasta vanhimpaan
$table->sortable(true, 'released', SORT_DESC);
$table->collapsible(false);
$table->initialbars(false);

$table->set_attribute('cellspacing', '0');
$table->set_attribute('class', 'generaltable generalbox');
$table->set_attribute('width', '100%');

// Ei sortausta kirjoittajan mukaan kun näkyvissä vain yhden kirjoittajan tekstejä
if($userid){
    $table->no_sorting('author');
}

$table->setup();

$posts = netforum_get_posts_for_list($netforum->id, $userid);

if($posts !== false){

     
    // Luodaan järjestämisessä käytettävät sarakeversiot datasta
    // Sarakeversioita tarvitaan jotta riviversio saadaan järjestettyä
    $postdata = array(
        "subject"  => array(),
        "userid"   => array(),
        "author" => array(),
        "released" => array(),
        "children" => array(),
        "parents" => array(),
        "id" => array(),
    	"reads" => array(),
        "grade" => array()//Grade on joka tapauksessa mukana

    );

    // Haetaan arvosanaskaala ennen looppiin menoa
    // ettei tarvitse joka viestille hakea uudestaan OPTIMOINTIA!
    $scale = make_grades_menu($netforum->scale);

    foreach ( $posts as $row ) {
        $postdata["subject"][] = $row->subject;
        $postdata["userid"][] = $row->userid;
        $postdata["author"][] = $row->lastname . " " .  $row->firstname ;
        $postdata["released"][] = $row->released;
        $postdata["children"][] = $row->children;
        // Lisätään vanhempien määrä rivi- ja sarakemuotoiseen dataan
        $row->parents = netforum_count_parents($row->id);
        $postdata["parents"][] = $row->parents;
        $postdata["id"][] = $row->id;


        // Lisätään lukukertojen määrä rivi- ja sarakemuotoiseen dataan
        $tmpread = get_record_sql("SELECT SUM(times) AS times FROM {$CFG->prefix}netforum_read WHERE postid = {$row->id}");

        if($tmpread->times){
            $row->reads = intval($tmpread->times);
        } else {
            $row->reads = 0;
        }
        $postdata["reads"][] = $row->reads;
        
        // Lisätään arvosanadata rivi ja sarakemuotoiseen dataan
        //jos on arvosteluoikeudet tai oma viesti
        if(has_capability('mod/netforum:canviewrate',$context) || $row->userid === $USER->id){
            $row->grade = netforum_get_ratings_mean($row->id, $scale);
            if($row->grade == '')
                $row->grade = $strnorate;
        }else{
            $row->grade = '';
        }
        $postdata["grade"][] = $row->grade;

    }

    // Haetaan taulukosta käyttäjän määräämät järjestysehdot
    $tablesort = $table->get_sql_sort();

    // parsitaan järjestämiskenttien nimet ja suunnat kirjaston antamasta sql-pätkästä
    $sorts = explode(",",trim($tablesort));

    if ($tablesort and is_array($sorts)) {
        $sortindex = array();
        $sortorder = array();
        // sortindex = järjestysotsikot
        // sortorder = järjestyssuunnat
        foreach ($sorts as $sort) {
            $data = explode(" ",trim($sort));
            $sortindex[] = trim($data[0]);
            $s = trim($data[1]);
            if ($s=="ASC") {
                $sortorder[] = SORT_ASC;
            } else {
                $sortorder[] = SORT_DESC;
            }
        }
        // oletusjärjestys jos järjestystä ei saada parsittua
        // tulee aina tulee olla kaksi järjestysehtoa ja
        // moodle ei välttämättä viitsi asettaa defuja
        $sortindex[] = 'released';
        $sortorder[] = SORT_ASC;
        $sortindex[] = 'subject';
        $sortorder[] = SORT_ASC;

        // asetetaan käytettävät järjestysparametrit
        // osoittamaan vastaaviin sarakemuotoisiin datakentiin
        $index1 = $postdata[$sortindex[0]];
        $index2 = $postdata[$sortindex[1]];
        // järjestetään:
        // multisort tekee mitä halutaan: $posts järjestyy ensisijaisesti index1 ja toissijaisesti index2:n mukaan
        array_multisort($index1, $sortorder[0], $index2, $sortorder[1], $posts);

    } // Nyt $posts on halutussa järjestyksessä
     

    // Taulukon sivutus
    $pagesize = 25;
    $table->pagesize($pagesize, count($posts));
    $start = $table->get_page_start();
    $pageposts = array_slice($posts, $start, $pagesize);


    // Generoidaan taulukkoon nyt näytettävä sivu
    foreach ( $pageposts as $row ) {
        // täydessä listassa käyttäjien nimet toimivat linkkeinä käyttäjäkohtaisiin listoihin
        $tmpuser = new object();
        
        $tmpuser->firstname = $row->firstname;
        $tmpuser->lastname = $row->lastname;
        if(!$userid){
            $author = "<a href=\"{$currenturl}&amp;userid={$row->userid}\">".fullname($tmpuser). "</a>";
        } else {
            $author = fullname($tmpuser);
        }

        // Korostukset
        $class = 'class="' . netforum_highlight($row->id, $row->userid) .'"';


        // Otsake on linkki yksittäiseen postiin
        $link = "<a href=\"post.php?id={$id}&amp;post=$row->id\" {$class}>$row->subject</a>";

        // Lisätään datarivi taulukkoon
        if(!isguestuser()){
            $table->add_data(array ($link, $author, userdate($row->released), $row->children, $row->parents, $row->reads, $row->grade  ));
        } else {
            $table->add_data(array ($link, $author, userdate($row->released), $row->children, $row->parents, $row->reads  ));
        }
    }
}//$posts !== false




// Tulostetaan varsinainen sisältö
echo '<table id="mod-netforum-main"><tr><td>';
print_heading_with_help($strlist, 'list', 'netforum');
$table->print_html();
if($userid !== false){
    echo "<p class=\"mod-netforum-center\"><a href=\"{$currenturlall}\">{$strshowall}</a></p>";
}
echo '</td><td id="mod-netforum-rightside">';

// ja sivulohkot
netforum_print_newest($netforum->id);
netforum_print_most_popular($netforum->id);
echo '</td></tr></table>';

?>