<?php

// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Lähteiden hallinta (omaan välilehteen)
 *
 * @author    Max Pagels
 * @package   mod-netforum
 * @copyright 2009 Helsingin Yliopisto, Tietojenkäsittelytieteen laitos
 * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

    require_once($CFG->libdir.'/tablelib.php');
    require_once ('lib.php');
    
    $citationschecked = optional_param('citations', array(), PARAM_INT);
    
    global $USER;
    $notice = ''; // ilmoitus, onnistuiko poisto vai ei
    $err_notice = '';
    
    //hae checkall/checknone JavaScript
    echo netforum_get_checkbox_javascript();
    
    $context = get_context_instance(CONTEXT_MODULE, $cm->id);
    
    if(!has_capability('mod/netforum:managecitations', $context)) {
        notice(get_string('noviewcitationspermission', 'netforum'));
    }
	
    if(data_submitted() and confirm_sesskey() and !empty($citationschecked)) {
        if(netforum_citation_delete($citationschecked)) {
            $notice = get_string('deleted', 'netforum');
        } else {
            $err_notice = get_string('couldnotdelete', 'netforum');
        }
        if($err_notice) {
            notify($err_notice, 'red');
        }
        if($notice) {
            notify($notice, 'green');
            add_to_log($course->id, "netforum",
                       "delete citation",
                       "view.php?id=$cm->id&amp;tab=".NETFORUM_TAB3."",
                       "$netforum->id");
        }
    }
	
	// alustetaan uusi taulukko, johon tulee sarakkeet
	// mm. aiheelle ja kirjoittajalle 
    $table = new flexible_table('mod-netforum-usercitations');
    $baseurl = $CFG->wwwroot.'/mod/netforum/view.php?id='.$id.'&amp;tab='.NETFORUM_TAB3.'';

    $tablecolumns = array('checkbox', 'subject', 'author');
    $tableheaders = array('', get_string('subject', 'netforum'),
                              get_string('author', 'netforum'));


    $table->define_columns($tablecolumns);
    $table->define_headers($tableheaders);
    $table->define_baseurl($baseurl);
    $table->sortable(true);
    $table->no_sorting('checkbox');

    $table->set_attribute('cellspacing', '0');
    $table->set_attribute('id', 'usercitations');
    $table->set_attribute('class', 'generaltable generalbox');
    $table->set_attribute('width', '100%');
    $table->column_style('checkbox', 'width', NETFORUM_CHECKBOX_WIDTH);
    
    $table->setup();

    // jos käyttäjä järjestää tietyn sarakkeen mukaan
    if ($table->get_sql_sort()) {
        $sort = 'ORDER BY ' . $table->get_sql_sort();
    } else {
        $sort = '';
    }
    
    // kyselyllä haetaan kaikki tarvittavat tiedot taulukon
    // täyttämistä varten
    $query = "SELECT p.id as pid, n.id as id, p.subject as subject, " . 
              sql_concat_join("' '", array('u.lastname', 'u.firstname')) . " as author
              FROM {$CFG->prefix}netforum_post p,
                   {$CFG->prefix}netforum_note n,
                   {$CFG->prefix}user u
              WHERE n.userid=$USER->id
              AND n.postid=p.id
              AND p.netforumid=$netforum->id
              AND n.link=1
              AND p.userid=u.id " . $sort;
    
    // montako riviä tuloksen sisältyy (käytetään sivuissa)       
    $totalcount = count_records_sql("SELECT COUNT(DISTINCT n.id)
                                     FROM {$CFG->prefix}netforum_note n,
                                          {$CFG->prefix}netforum_post p
                                     WHERE n.userid=$USER->id
                                     AND n.postid=p.id
                                     AND p.netforumid=$netforum->id
                                     AND n.link=1");
              
    $table->initialbars(true);
    
    // montako riviä näytetään per sivu
    $table->pagesize(NETFORUM_DEFAULT_PAGE_SIZE, $totalcount);
    
    print_heading_with_help(get_string('thirdtabname', 'netforum'), 'citations', 'netforum');
              
    // jos kysely palauttaa tietoja, vie tiedot oikeisiin kohtiin
    // taulukossa ja lisää hallintaan liittyvät nappulat
    if ($records = get_records_sql($query, $table->get_page_start(), $table->get_page_size())) {   
        echo '<form class="citation-management-form" method="post" action="'.$baseurl.'"><div>';
        
        $citations = array_values($records);
        
        foreach($citations as $citation) {
            $author = $citation->author;
            $subject = "<a href=\"post.php?id=$id&amp;post=$citation->pid\">$citation->subject</a>";
            $checkbox = '<input type="checkbox" name="citations[]" value="'.$citation->id.'" />';
            $data = array($checkbox, $subject, $author);
            $table->add_data($data);
        }
        
        echo '<div style="text-align:right;">
              <input type="button" onclick="checkall(\'citations[]\')" value="'.get_string('selectall', 'netforum').'" /> ';
        echo '<input type="button" onclick="checknone(\'citations[]\')" value="'.get_string('deselectall', 'netforum').'" /> ';
        echo '<input type="hidden" name="sesskey" value="'.sesskey().'" /> ';
        echo '<button id="citations-management-submit" type="submit">'.get_string('deletecitations', 'netforum').'</button>
              <br /><br /></div>';
              
        // tulosta valmis taulukko
        $table->print_html();
        echo '</div></form><br/>';
    } else {
        // jos kysely epäonnistui tai tuloksia ei ollut, ilmoita
        // siitä käyttäjälle
        $table->print_html();
    }

?>