<?php

// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Netforum-moduulin asennuslomake
 *
 * @author    Max Pagels
 * @package   mod-netforum
 * @copyright 2009 Helsingin Yliopisto, Tietojenkäsittelytieteen laitos
 * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

require_once($CFG->dirroot.'/course/moodleform_mod.php');

class mod_netforum_mod_form extends moodleform_mod {

    /**
    * Tämä funktio piirtää netforum-moduulin asennuslomakkeen
    *
    * @author Max Pagels
    */
    function definition() {

        global $CFG;
        $mform =& $this->_form;

        // lisätään yleinen form-legend
        $mform->addElement('header', 'general', get_string('general', 'netforum'));

        // nimi-kenttä, menee netforum-taulun "name"-kenttään 
        $mform->addElement('text', 'name', get_string('netforumname', 'netforum'), array('size'=>'64'));

        // jos konfiguraatiotiedostossa on määritelty, että html-tägejä EI
        // sallita, siivotaan ne pois
        if (!empty($CFG->formatstringstriptags)) {
            $mform->setType('name', PARAM_TEXT);
        } else {
            $mform->setType('name', PARAM_CLEAN);
        }

        // Tehdään validointi sekä clientillä että serverillä - kentän
        // maksimissaan 255 merkkiä
        $mform->addRule('name', null, 'required', null, 'client');
        $mform->addRule('name', get_string('maximumchars', 'netforum'), 'maxlength', 255, 'client');

        // intro-kenttä, menee netforum-taulun "intro"-kenttään
        $mform->addElement('htmleditor', 'intro', get_string('intro', 'netforum'));
        $mform->setType('intro', PARAM_RAW);
        $mform->addRule('intro', null, 'required', null, 'client');

        // Valitaan, arvostellaanko viestit vai ei, ja jos arvostellaan, niin miten
        $mform->addElement('header', '', get_string('grade', 'netforum'));
        $mform->addElement('select', 'assessed', get_string('aggregatetype', 'netforum') , netforum_get_aggregate_types());
        $mform->setDefault('assessed', 0);
        $mform->addElement('modgrade', 'scale', get_string('grade', 'netforum'), false);
        $mform->disabledIf('scale', 'assessed', 'eq', 0);

        // Rajoitetaan milloin viestejä voi arvostella
        $mform->addElement('checkbox', 'ratingtime', get_string('ratingtime', 'netforum'));
        $mform->disabledIf('scale', 'assessed', 'eq', 0);
        $mform->addElement('date_time_selector', 'assesstimestart', get_string('from'));
        $mform->disabledIf('assesstimestart', 'assessed', 'eq', 0);
        $mform->disabledIf('assesstimestart', 'ratingtime');

        $mform->addElement('date_time_selector', 'assesstimefinish', get_string('to'));
        $mform->disabledIf('assesstimefinish', 'assessed', 'eq', 0);
        $mform->disabledIf('assesstimefinish', 'ratingtime');

        $this->standard_coursemodule_elements();
        $this->add_action_buttons();

    }
    
    /**
    * Kun asennuslomake on lähetetty tämä funktio tarkistaa, etteivät skaala-,
    * arviointi- ja aikaasetukset ole keskenään ristiriidassa (ja korjaa
    * ristiriidat jos niitä löytyy
    *
    * @author Max Pagels
    */
    function data_preprocessing(&$default_values) {
        if (empty($default_values['scale'])) {
            $default_values['assessed'] = 0;
        }
        
        if (empty($default_values['assessed'])) {
            $default_values['ratingtime'] = 0;
        }
        else {
            $default_values['ratingtime'] = ($default_values['assesstimestart'] && $default_values['assesstimefinish']) ? 1 : 0;
        }
    }
    
}

?>