<?php

// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.


/**
 * Generoi naapurigraafin 
 * 
 * Generoi yksittäiselle viestille naapurigraafin joka näyttää
 * viestin, sen lapset ja vanhemmat sekä kaikkien näiden viestien
 * väliset viittaussuhteet. Graafi generoidaan dotilla ja esitetään
 * kuvana. 
 *
 * @package   mod-netforum
 * @copyright 2009 Helsingin Yliopisto, Tietojenkäsittelytieteen laitos
 * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

global $CFG;


require_once ('lib.php');
require_once (dirname(dirname(dirname(__FILE__))).'/config.php');

$pid = required_param('id', PARAM_INT); 
$format = required_param('fmt', PARAM_ALPHA);
$shrink = optional_param('shrink', false, PARAM_BOOL);

$id = netforum_get_cmid(netforum_get_id($pid));

if ($id) {
    /// Haetaan coursemodule id:n perusteella
    if (! $cm = get_coursemodule_from_id('netforum', $id)) {
        error(get_string('cmid_incorrect', 'netforum')); // 'Course Module ID was incorrect'
    }
    /// coursemodulista haetaan course-olio
    if (! $course = get_record('course', 'id', $cm->course)) {
        error(get_string('misconfigured', 'netforum')); // 'Course is misconfigured'
    }

    /// coursemodulista haetaan netforum-olio
    if (! $netforum = get_record('netforum', 'id', $cm->instance)) {
        error(get_string('cm_incorrect', 'netforum')); // 'Course module is incorrect'
    }

} else {
    error('INTERNAL LOGICAL ERROR'); // 'You must specify a course_module ID'

}

require_login($course);

add_to_log($course->id, "netforum", "view neighbourgraph", "neighbourgraph.php?id=$cm->id&amp;fmt=$format&amp;shrink=".intval($shrink),
             "$netforum->id");



// Jos polkua ei ole asetettu herjataan
if(!isset($CFG->netforum_dotdir)){
    error(get_string('dotdir_unset', 'netforum'));
}

// Jos polku on asetettu tyhjäksi ei tehdä mitään
if($CFG->netforum_dotdir === ''){
    exit;
}

// Valitaan oikea mime
if( $format == 'pdf' ){
    $fmt = 'pdf';
    $header = 'Content-Type: application/pdf';
} else if ( $format == 'png' ) {
    $fmt = 'png';
    $header = 'Content-Type: image/png';
} else if ( $format == 'map' ) {
    $fmt = 'cmapx';
    $header = 'Content-Type: text/plain';
} else if ( $format == 'source' ) {
    $fmt = 'source';
    $header = 'Content-Type: text/plain';
} else {
    error(get_string('unknown_imagetype', 'netforum'));
}
//$header = 'Content-Type: text/plain';
header($header);

netforum_generate_neighbour_web($fmt, $pid, false, $shrink );

?>