<?php

// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Käyttäjän henkilökohtaisten kommenttikohtaisten 
 * muistiinpanojen näyttämisestä vastaava luokka.
 *
 * @package   mod-netforum
 * @copyright 2009 Helsingin Yliopisto, Tietojenkäsittelytieteen laitos, Modex
 * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

require_once('../../config.php');
require_once('lib.php');
require_once('note_form.php');
//require_once('note_lib.php');

global $CFG; 
global $USER;

$netforumid   	= required_param('netforumid', PARAM_INT);
$noteID			= optional_param('noteid', NULL, PARAM_INT);
$postID			= optional_param('postid', NULL, PARAM_INT);

//muuttujia, joiden avulla pidetään selvillä saatiinko noteid tai postid postissa(tai getissä)
$noteIDGiven = false;
$postIDGiven = false;
    
if(isset($noteID)){
         $noteIDGiven = true;
}
if(isset($postID)){
         $postIDGiven = true;
}
    
//jos postid ei ollut entuudestaan selvillä, haetaan se
if(! $postIDGiven){
    $postID = netforum_get_postid_from_note($noteID);
}
    

//haetaan kurssista ja moduulista dataa
if ($netforumid) {
    /// Haetaan coursemodule id:n perusteella
    if (! $cm = get_coursemodule_from_id('netforum', $netforumid)) {
        error('Course Module ID was incorrect');
    }
    /// coursemodulista haetaan course-olio
    if (! $course = get_record('course', 'id', $cm->course)) {
        error('Course is misconfigured');
    }
    /// coursemodulista haetaan netforum-olio
    if (! $netforum = get_record('netforum', 'id', $cm->instance)) {
        error('Course module is incorrect');
    }
} 
else {
    error('You must specify a course_module ID');
}

    

//vaaditaan, että on sisäänkirjautunut TODO: tarkista toimiiko..
require_login($course, true, $cm);
$context = get_context_instance(CONTEXT_MODULE, $cm->id);

//tarkista, että käyttäjä voi kirjoittaa ja muokata kommentteja
if(! has_capability('mod/netforum:writecomments', $context)){
    error(get_string('modifyingnotesdenied', 'netforum'));
    exit;
}
    
$strnetforums = get_string('modulenameplural', 'netforum');
$strnetforum  = get_string('modulename', 'netforum');


//generoidaan polkupalkki
$navlinks = array();
$navlinks[] = array('name' => $strnetforums, 'link' => "index.php?id=$course->id", 'type' => 'activity');
$navlinks[] = array('name' => format_string($netforum->name), 'link' => "view.php?id={$netforumid}", 'type' => 'activityinstance');
$navlinks[] = array('name' => get_string('postui', 'netforum'), 'link' => "post.php?id={$netforumid}&post={$postID}", 'type' => 'activityinstance');
$navlinks[] = array('name' => get_string('note', 'netforum'), 'link' => '', 'type' => 'activityinstance');

$navigation = build_navigation($navlinks);

/// Tulostetaan sivun otsakkeet
print_header_simple(format_string($netforum->name), '', $navigation, '', '', true,
update_module_button($cm->id, $course->id, $strnetforum), navmenu($course, $cm));
echo '<table id="mod-netforum-main"><tr><td>';


//poistetaanko
if(isset($_POST['delete'])){
/*    if(! has_capability('mod/netforum:writecomments', $context)){
        error(get_string('noupdatenotespermission', 'netforum'));
        exit;
    }*/
//    add_to_log($course->id, "netforum", "note_delete", "post.php?id=$cm->id&post=$postid", "$netforum->id");
    add_to_log($course->id, "netforum", "delete note");
    $postID = netforum_get_postid_from_note($noteID);
    netforum_note_delete($noteID);
    $message = '<p>'.get_string("notedeleted", "netforum") . '</p>';
    redirect("post.php?id={$netforumid}&post={$postID}", $message, 2);
    exit;
}

//jos postid oltiin saatu parametrina, kutsutaan formia sen kanssa 
//(eli note ei ole olemassa -> halutaan luoda se)
if($postIDGiven){
    $mform_post = new mod_netforum_note_form('note.php?netforumid='.$netforumid.'&postid='.$postID);
}
//jos noteid oltiin saatu parametrina, kutsutaan formia sen kanssa 
//(eli note on jo olemassa -> halutaan muokata sitä)
else if($noteIDGiven){
    $mform_post = new mod_netforum_note_form('note.php?netforumid='.$netforumid.'&noteid='.$noteID);
}


if ($fromform = $mform_post->get_data()) {

    $fromform->netforumid    = $netforum->id;

    //jos ollaan luomassa uutta notea
    if($postIDGiven && !$noteIDGiven){
        
        if($fromform->action == "write" && !netforum_get_noteid($postID, $USER->id)) {
//      if(!netforum_note_add($postID, $USER->id, $fromform->comment, $fromform->link)) {
            /*if(! has_capability('mod/netforum:writecomments', $context)){
                error(get_string('noupdatenotespermission', 'netforum'));
                exit;
            }*/
            if(!netforum_note_add($postID, $USER->id, $fromform->comment)) {
                print_error("couldnotwrite", "netforum");
                exit;
            }
            add_to_log($course->id, "netforum", "add note", "note.php?netforumid=$netforumid&postid=$postID");
            //redictia noten viestiin
            $message = '<p>'.get_string("noteadded", "netforum") . '</p>';
            redirect("post.php?id={$netforumid}&post={$postID}", $message, 2);
	        exit;
        }
    }
        
    //jos ollaan muokkaamassa vanahaa notea
    else if(!$postIDGiven && $noteIDGiven){ 
//      if($fromform->action == "edit" && isset($_POST['noteid'])) {
//      $postID = netforum_get_postid_from_note($noteID);
		
        if($fromform->action == "edit") {
//          if(!netforum_note_edit($noteID, $fromform->comment, $fromform->link)) {
/*            if(! has_capability('mod/netforum:writecomments', $context)){
                error(get_string('noupdatenotespermission', 'netforum'));
                exit;
            }*/
            if(!netforum_note_edit($noteID, $fromform->comment)) {
                print_error("couldnotwrite", "netforum");
                exit;
            }
	        add_to_log($course->id, "netforum", "update note", "note.php?netforumid=$netforumid&noteid=$noteID");
	        //redictia noten viestiin
            $message = '<p>'.get_string("noteedited", "netforum") . '</p>';
            redirect("post.php?id={$netforumid}&post={$postID}", $message, 2);
            exit;
        }

    }
    else{
        //noteid ja postid ei täsmää tai jotain..
        //TODO: ehkä vois poistaa tekstin ja laittaa vaan redirectin
        error(get_string("notemisarrival", "netforum"));
    }
}




//viestin kirjoittamiseen
//if(!isset($_POST) || isset($_POST['write'])) {
//if(!isset($_POST['edit']) && isset($_POST['write'])) {
if(!isset($_POST['edit'])) {
    //luo uusi post object
    $tmp_note = new object();
    $tmp_note->comment = '';
    $tmp_note->link = 0;
    $tmp_note->mode = 'write';
}

//viestin muokkaaminen (jos oma)
else if( isset($_POST['edit'])) {
//else if($action == 'edit'){
    
    if(!($note = netforum_note_get($noteID, $USER->id))){
        error("notea ei ollut tai jotain..");
    }
        
    $tmp_note = new object();
    $tmp_note->comment = $note->comment;
    $tmp_note->link = $note->link;
    $tmp_note->mode = 'edit';
}
    
//if( isset($_POST['write']) || isset($_POST['edit']) || !isset($_POST) ) {
if(true){ //??
//if($action == 'write' || $action == 'add_new_note' || $action == 'edit'){
    
    $mform_post->set_data(array('comment'=>$tmp_note->comment,
                                'link'=>$tmp_note->link,
                                'action'=>$tmp_note->mode));
    
    //tulostetaan alkuperäinen viesti
    //paitsi ettei tulostetakaan koska sen funktiot on yhä viewissä -> eli niihin ei pääse käsiksi
    //netforum_post_view($netforum, netforum_get_post($postid), $course, $cm);
    //	echo 'Placeholder alkuperäiselle postaukselle';

    //    if(!isset($postID){
    //    if($postID == -1){
    //        $postID = get(netforum_get_postid_from_note($noteID));
    //    }
    netforum_post_view($netforum, netforum_get_post($postID), $course, $cm);
    
    $mform_post->display();
    
    //muistutus siitä kenelle kommentit näkyvät
    echo '<p class="mod-netforum-center">'.get_string('remembernotevisibility','netforum');
    helpbutton('note', get_string('note', 'netforum'), 'netforum');
    echo '</p>';
}

// ja sivulohkot
echo '</td><td id="mod-netforum-rightside">';
netforum_print_newest($netforum->id);
netforum_print_most_popular($netforum->id);
echo '</td></tr></table>';

/// Sivun varsinainen sisältö päättyy  
print_footer($course);

?>