<?php 
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 *Tarkistetaan oikeudet ja tallennetaan arvosana tietokantaan ja pivitetn arviointikirjaa
 *
 */

    
    require_once('../../config.php');
    require_once('lib.php');

    $netforumid = required_param('netforumid', PARAM_INT); // netforum josta arvioitavat viestit ovat
    //tarkistetaan netforum id
    if (!$netforum = get_record('netforum', 'id', $netforumid)) {
        error("Course ID was incorrect");
    }
    //tarkistetaan kurssi id
    if (!$course = get_record('course', 'id', $netforum->course)) {
        error("Course ID was incorrect");
    }
    //kurssimoduuli
    if (!$cm = get_coursemodule_from_instance('netforum', $netforum->id)) {
        error("Course Module ID was incorrect");
    } else {
        $netforum->cmidnumber = $cm->id; //MDL-12961
        }
    //vaaditaan kirjautuminen
    require_login($course, false, $cm);
    // jos vieraskyttj niin hyltn
    if (isguestuser()) {
        error("Guests are not allowed to rate entries.");
    }
    // Onko arviointi pll
    if (!$netforum->assessed) {
        error("Rating of items not allowed!");
    }
    //tarkistetaan oikeudet arvioitiin
    $context = get_context_instance(CONTEXT_MODULE, $cm->id);
    require_capability('mod/netforum:canrate', $context);
    //haetaan vlitettvt tiedot
    if ($data = data_submitted()) {
    
        foreach ((array)$data as $postid => $rating) {
            if (!is_numeric($postid)) {
                continue;
            }
           

            // haetaan tietokannasta arvioitavan postin tiedot
           $sql = "SELECT *
                      FROM {$CFG->prefix}netforum_post
                     WHERE id = '$postid'";

            if (!$post = get_record_sql($sql)) {
                error("Incorrect postid - $postid");
            }
            //tarkistetaan onko arviointiaika pll ja onko viesti julkaistu sen sisll
            if ($netforum->assesstimestart and $netforum->assesstimefinish) {
                if ($post->released < $netforum->assesstimestart or $post->released > $netforum->assesstimefinish) {
                    // we can not rate this, ignore it - this should not happen anyway unless teacher changes setting
                    continue;
                }
            }
            //jos lhetetn ei arviointia, poistetaan kyseisen arvostelijan vanha arviointi tietokannasta
            if ($rating == NETFORUM_UNSET_POST_RATING) {
                delete_records('netforum_ratings', 'postid', $postid, 'userid', $USER->id);
                netforum_update_grades($netforum, $post->userid);
                //redirect("$CFG->wwwroot/mod/netforum/post.php?id=$cm->id&post=$postid", get_string('ratingssaved', 'netforum'));
            //jos on vanha arviointi pivitetn se
            } else if ($oldrating = get_record('netforum_ratings', 'userid', $USER->id, 'postid', $postid)) {
                if ($rating != $oldrating->rating) {
                    $oldrating->rating = $rating;
                    $oldrating->time   = time();
                    if (! update_record('netforum_ratings', $oldrating)) {
                        error("Could not update an old rating ($post->id = $rating)");
                    }
                    netforum_update_grades($netforum, $post->userid);
                    add_to_log($course->id, "netforum", "update rating", "$CFG->wwwroot/mod/netforum/post.php?id=$cm->id&post=$postid", "$netforum->id");
                }
            //viedn kokonaan uusi arviointi tietokantaan
            } else {
                $newrating = new object();
                $newrating->postid  = $post->id;
                $newrating->userid = $USER->id;
                $newrating->time   = time();
                $newrating->rating = $rating;
                if (!insert_record('netforum_ratings', $newrating)) {
                    error("Could not insert a new rating ($postid = $rating)");
                }
                netforum_update_grades($netforum, $post->userid);
                add_to_log($course->id, "netforum", "add rating", "$CFG->wwwroot/mod/netforum/post.php?id=$cm->id&post=$postid", "$netforum->id");
            }
        }

        if ($postid) {
            redirect("$CFG->wwwroot/mod/netforum/post.php?id=$cm->id&post=$postid", get_string('ratingssaved', 'netforum'));
        } 

    } else {
        error("This page was not accessed correctly");
    }

?>