<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 *Haetaan tietokannasta tilastotietoja opiskelijoista
 *ja tulostetaan taulukko kaikista instanssin opiskelijoista.
 *Tämä sivu on tarkoitettu opetuksen tueksi opettajille.
 */

require_once ("lib.php");
require_once($CFG->libdir.'/tablelib.php');

global $USER;

$context = get_context_instance(CONTEXT_MODULE, $cm->id);

//testataan käyttäjän oikeudet
if (!has_capability('mod/netforum:viewstatistic',$context))
    error('Not allowed');

// Sivun url + perusparametrit KOVAKOODATTU
$current_url =  "view.php?id={$id}&amp;tab=" . NETFORUM_TAB6 .  "";
$current_url_all =  "view.php?id={$id}&amp;tab=" . NETFORUM_TAB6 .  "";
$moveto_url = "view.php?id={$id}&amp;tab=" . NETFORUM_TAB2 .  "";

//lokalisaatiot
$student = get_string('studentname', 'netforum');
$posts = get_string('numberofposts', 'netforum');
$graded = get_string('graded', 'netforum');
$times = get_string('timesread', 'netforum');
$nchildren = get_string('numberofchildren', 'netforum');
$nparents = get_string('numberofparents', 'netforum');
$grade = get_string('avggrade', 'netforum');
$strstatistic  = get_string('sixthtabname', 'netforum');

//taulukon otsikot ja muuttujat taulukoina
$tablecolumns = array( 'studentname', 'numberofposts', 'graded', 'timesread', 'nchildren', 'nparents', 'grade');
$tableheaders = array($student, $posts, $graded, $times, $nchildren, $nparents, $grade);

//luodaan flexible_table
$table = new flexible_table('mod-netforum-view-statistic');

//määritellään taulukon rivit ja sarakkeet ja url
$table->define_columns($tablecolumns);
$table->define_headers($tableheaders);
$table->define_baseurl($CFG->wwwroot.'/mod/netforum/'.$current_url);

//flexible_tablen perusparametrit
$table->sortable(true);
$table->collapsible(false);
$table->initialbars(false);

$table->set_attribute('cellspacing', '0');
$table->set_attribute('class', 'generaltable generalbox');
$table->set_attribute('width', '100%');

$table->setup();

//Määritellään taulukon järjestäminen
if ($table->get_sql_sort()) {
        $sort = 'ORDER BY ' . $table->get_sql_sort();
    }
    else {
        $sort = '';
}
                                
//varsinaisen tietokantakyselyn määrittely
$query = "SELECT u.id id, ". sql_concat_join("' '", array('u.lastname', 'u.firstname')) . " studentname, 
(SELECT COUNT(p2.userid) FROM {$CFG->prefix}netforum_post p2 WHERE p2.userid=u.id AND p2.netforumid=$netforum->id AND p2.released != 0) numberofposts, 
(SELECT COUNT(ra.userid) FROM {$CFG->prefix}netforum_post p2, {$CFG->prefix}netforum_ratings ra WHERE ra.userid=$USER->id AND p2.id=ra.postid AND p2.userid=u.id AND p2.netforumid=$netforum->id) graded, 
(SELECT SUM(r.times) FROM {$CFG->prefix}netforum_read r, {$CFG->prefix}netforum_post p2 WHERE p2.userid=u.id AND p2.id=r.postid AND p2.netforumid=$netforum->id) timesread, 
(SELECT COUNT(w.child) FROM {$CFG->prefix}netforum_web w, {$CFG->prefix}netforum_post p2 WHERE p2.userid=u.id AND w.parent=p2.id AND p2.netforumid=$netforum->id) nchildren, 
(SELECT COUNT(w.parent) FROM {$CFG->prefix}netforum_web w, {$CFG->prefix}netforum_post p2 WHERE p2.userid=u.id AND w.child=p2.id AND p2.netforumid=$netforum->id) nparents, 
(SELECT AVG(ra.rating) FROM {$CFG->prefix}netforum_ratings ra, {$CFG->prefix}netforum_post p2 WHERE p2.userid=u.id AND p2.netforumid=$netforum->id AND p2.id=ra.postid) grade
              FROM {$CFG->prefix}netforum_post p,
                   {$CFG->prefix}user u
              WHERE p.netforumid=$netforum->id
              AND p.userid=u.id ". $sort;
              

//suoritetaan tietokantakysely jos ei onnistu tai tietokanta on tyhjä tulostetaan tyhjä sivu
if(!$records = get_records_sql($query)) {
    print_heading_with_help($strstatistic, 'statistic', 'netforum');
    $table->print_html();

//luodaan taulukkoon sisältö tietokantakyselyn tuloksesta
} else {
    foreach($records as $row) {
	    $author = "<a href=\"{$moveto_url}&amp;userid={$row->id}\">".$row->studentname. "</a>";
        $table->add_data(array($author, $row->numberofposts, $row->graded, $row->timesread, $row->nchildren, $row->nparents, $row->grade));
    }
//tulostetaan varsinainen taulukko.
print_heading_with_help($strstatistic, 'statistic', 'netforum');
$table->print_html();
}
