<?php

// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.


/**
 * Tulostaa yksittäisen netforumilmentymän kaikista viesteistä otsikkopilven
 * 
 * Netforum-moduulin päänäkymä. Otsikkopilvi on tägipilven kaltainen
 * visualisaatio kaikista/osasta netforum-ilmentymään liittyvistä viesteistä.
 * Otsikkopilvessä näkyy otsikkojen alkuosat siten että enemmän yhteyksiä
 * keränneet viestit ovat suuremmalla kirjasimella. Toisaalta viestit ovat
 * pilvessä uutuusjärjestyksessä. Näin käyttäjä näkee pilvestä suoraan kaksi
 * asiaa: mikä ovat uusimmat viestit ja mitkä ovat »parhaimmat» viestit. 
 *
 * @package   mod-netforum
 * @copyright 2009 Helsingin Yliopisto, Tietojenkäsittelytieteen laitos
 * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */


/**
 * Tulostaa yksittäisen netforumilmentymän kaikista viesteistä otsikkopilven
 *
 * @author Juho Kilpikoski
 */
require_once ("lib.php");


$hide = optional_param('hide', false, PARAM_BOOL);

$baseurl =  "view.php?id={$id}&amp;tab=" . NETFORUM_TAB1 .  "";

// Lokalisaatio
$strcloud  = get_string('tagcloud', 'netforum');
$strlevel5visible  = get_string('level5visible', 'netforum');
$strlevel5hidden  = get_string('level5hidden', 'netforum');
$strnomessages = get_string('no_messages', 'netforum');

// Generoidaan pilvi-html
$posts = netforum_get_posts($netforum->id);
if($posts !== false){
    $cloudhtml = print_heading_with_help($strcloud, 'tagcloud', 'netforum', '', true);
    $cloudhtml .=  print_box_start('generalbox', '', true);

    $cloudhtml .= '<div id="mod-netforum-tagcloud">';

    $maxadjacent = netforum_count_most_connected($netforum->id);

    // Generoidaan yksittäisten viestien html-oliot
    foreach ( $posts as $row ) {
        // Korostukset
        $class = 'class="' . netforum_highlight($row->id, $row->userid) .'"';
         
        // linkkirunko käyttäjäkohtaisiin viestilistauksiin
        $userpostsurl =  "view.php?id={$id}&amp;tab=" . NETFORUM_TAB2 .  "&amp;userid=";


        // Käsitellään ylipitkät otsikot
        // jaetaan sanoihin
        $arrsubject = preg_split('/\s/', trim($row->subject));
        $strsubject = '';
        $truncated = false;

        foreach ( $arrsubject as $i => $word ){
            // jos sana on ensimmäinen lisätään
            if( $i == 0  ){
                $strsubject .= $word;
            } else if ((strlen($strsubject) + strlen($word)) < 24) { // jos sana mahtuu lisätään välin jäkeen
                $strsubject .= ' ' . $word;
            } else { // ei mahdu, tarvitaan katkaisua, lopetetaan
                $truncated = true;
                break;
            }
        }

        // lisätään pisteet jos katkaistiin
        if($truncated){
            $strsubject .= '…';
        }


        // Muodostetaan viestikohtainen html
        $cloudhtml .= '<div class="mod-netforum-wrapper"><div class="'. netforum_cloud_class($row->adjacent, $maxadjacent, $hide). 
                      ' mod-netforum-cloud-subject">';
        $cloudhtml .= "<a href=\"post.php?id={$id}&amp;post=$row->id\" {$class}>{$strsubject}</a>";
        $tmpuser = new object();
        
        $tmpuser->firstname = $row->firstname;
        $tmpuser->lastname = $row->lastname;
        
        $cloudhtml .= '</div><div class="mod-netforum-hidden"><h3>' . trim($row->subject) . '</h3><div>' .
                       fullname($tmpuser);

        $cloudhtml .= ' <a href="'.$userpostsurl . $row->userid . '" title="Listaa kaikki tämän viestin kirjoittajan kirjoittamat viestit">'.
                      'Kirjoittajan viestit</a></div><div>' . userdate($row->released) . '</div>';
        // TODO poimintaurli
        //$cloudhtml .= '<div><a href="">'.get_string('add_to_list', 'netforum') .'</a></div>';
        //$cloudhtml .= '<div>'.$row->adjacent .'</div>';
        $cloudhtml .= "</div></div>";
    }

    $cloudhtml .= '</div>' . print_box_end(true);
    if($hide){
        $cloudhtml .= "<p class=\"mod-netforum-center\"><a href=\"{$baseurl}\">{$strlevel5hidden}</a></p>";
    } else {
        $cloudhtml .= "<p class=\"mod-netforum-center\"><a href=\"{$baseurl}&amp;hide=1\">{$strlevel5visible}</a></p>";
    }
} else {
    $cloudhtml = "<p class=\"mod-netforum-center\">{$strnomessages}</p>";
}

// Tulostetaan varsinainen sisältö
echo '<table id="mod-netforum-main"><tr><td>';

echo $cloudhtml;

// ja sivulohkot
echo '</td><td id="mod-netforum-rightside">';
netforum_print_newest($netforum->id);
netforum_print_most_popular($netforum->id);
echo '</td></tr></table>';

?>