<?php

// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.


/**
 * Generoi päänäkymien sivujen rungot
 * 
 * Tulostaa headerit ja footerit päänäkymiin sekä tarkastaa kaikille
 * yhteiset parametrit ja hakee kaikkien tarvitesemat coursemodule,
 * course ja netforum-oliot. Tulostaa myös pääsivuille yheisen 
 * navigointipalkin.
 *
 * @package   mod-netforum
 * @copyright 2009 Helsingin Yliopisto, Tietojenkäsittelytieteen laitos
 * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

require_once (dirname(dirname(dirname(__FILE__))).'/config.php');
require_once ("lib.php");

/// Pakollinen parametri on couse_module id
$id = required_param('id', PARAM_INT); 

if ($id) {
    /// Haetaan coursemodule id:n perusteella
    if (! $cm = get_coursemodule_from_id('netforum', $id)) {
        error(get_string('cmid_incorrect', 'netforum')); // 'Course Module ID was incorrect'
    }
    /// coursemodulista haetaan course-olio
    if (! $course = get_record('course', 'id', $cm->course)) {
        error(get_string('misconfigured', 'netforum')); // 'Course is misconfigured'
    }

    /// coursemodulista haetaan netforum-olio
    if (! $netforum = get_record('netforum', 'id', $cm->instance)) {
        error(get_string('cm_incorrect', 'netforum')); // 'Course module is incorrect'
    }

} else {
    error(get_string('no_cmid', 'netforum')); // 'You must specify a course_module ID'
}


require_login($course, true, $cm);

/// Haetaan lokalisaatiot
$strnetforums = get_string('modulenameplural', 'netforum');
$strnetforum  = get_string('modulename', 'netforum');

/// Generoidaan »polkupalkki»
$navlinks = array();
$navlinks[] = array('name' => $strnetforums, 'link' => "index.php?id=$course->id", 'type' => 'activity');
$navlinks[] = array('name' => format_string($netforum->name), 'link' => '', 'type' => 'activityinstance');

$navigation = build_navigation($navlinks);

/// Tulostetaan sivun otsakkeet
print_header_simple(format_string($netforum->name), '', $navigation, '', '', true,
update_module_button($cm->id, $course->id, $strnetforum), navmenu($course, $cm));

//if (!empty($netforum->intro)) {
  //  print_box(format_text($netforum->intro), 'generalbox', 'intro');
//}



include_once('./tabs.php');


/// Sivun varsinainen sisältö päättyy  
print_footer($course);